"	***** Smalltalk chunk source generated June 12, 2000 at 5:54:44 pm *****	"!

"
The contents of this file are subject to the Mozilla Public
License Version 1.1 (the 'License'); you may not use this file
except in compliance with the License. You may obtain a copy of
the License at http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an 'AS
IS' basis, WITHOUT WARRANTY OF ANY KIND, either express or
implied. See the License for the specific language governing
rights and limitations under the License.

The Original Code is Messenger Support.

The Initial Developer of the Original Code is GemStone Systems, Inc.
Portions created by GemStone Systems, Inc. are
Copyright (C) 2000 GemStone Systems, Inc.  All
Rights Reserved.

Contributor(s):
	Paul Baumann <pbaumann@gemstone.com>

"!

"	***** Prerequisites *****	
		Kernel R4.00
"!

"	***** Included versions *****	"!
"		CstMessengerBaseExt 2.0		"!
"	CstMessengerSupport 2.8		"!

"	***** Class declarations *****	"!

Object subclass: #CstMessageDispatcher
	instanceVariableNames: 'handler '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CstMessengerSupport'!

CstMessageDispatcher class instanceVariableNames: 'singleton '!

Object subclass: #CstMessageHandler
	instanceVariableNames: 'owner '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CstMessengerSupport'!

CstMessageHandler subclass: #CstMessageHandlerVa
	instanceVariableNames: 'hostSignal hostPrompter '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CstMessengerSupport'!

CstMessageHandler subclass: #CstMessageHandlerVw
	instanceVariableNames: 'hostSignal hostPrompter '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CstMessengerSupport'!


"	***** Behavior declarations *****	"!

!CstMessageDispatcher methodsFor: 'commenting'!

basicComment: aString
	"Comments are the lowest priority of all messages. They may
	 be displayed in a window status area, or may be written to a
	 log. They never interrupt processing."

	^self handler reportComment: aString.
! !

!CstMessageDispatcher methodsFor: 'confirmation'!

basicConfirm: aString
	"Prompt the user in some way for a boolean response when asked aString."

	^self handler confirm: aString.! !

!Object methodsFor: 'CstMessengerBaseExt'!

cstMessenger
	"Answer something that knows how to handle messages in a 
	 portable way. This eliminates many method references to
	 a global and allows for easy substitution in the future."

	^CstMessageDispatcher singleton! !

!CstMessageDispatcher methodsFor: 'error reporting'!

basicError: aString
	"Raise an error signal with aString for the notifier string."

	^self handler reportError: aString.! !

!CstMessageDispatcher methodsFor: 'failure reporting'!

basicFailure: errorString
	"An assertion test has failed. This is considered a low priority type of error
	 condition and execution is normally allowed to continue unless debugging."

	^self handler reportFailure: errorString! !

!CstMessageDispatcher methodsFor: 'request string'!

basicRequest: aString default: defaultString
	"Prompt the user to enter a string or confirm the defaultString.
	 This normally suspends processing until an answer is given.
	 Normal behavior would be to return nil if a cancel button
	 is pressed."

	^self handler 
		request: aString 
		default: defaultString.! !

!CstMessageDispatcher methodsFor: 'choosing'!

choose: multilineMessage labels: labelArray values: returnValueArray default: defaultValue

	"Open a prompter to display the given message and a line of buttons.  Return the value
	 of the selected button, or the default value if the dialog is terminated by the user."

	"
		self messenger 
			choose: 'This is a test of the emergency broadcast notification system.' 
			labels: #('Yes' 'No' 'Maybe' 'Otherwise') 
			values: #(1 2 3 4) 
			default: 1.
	"

	^self handler
		choose: multilineMessage 
		labels: labelArray 
		values: returnValueArray 
		default: defaultValue! !

!CstMessageDispatcher methodsFor: 'commenting'!

commentAdd: aString with: arg1
	"Substitute all %1...%4 with the arguments passed."

	self 
		commentAdd: aString 
		withArguments: (Array with: arg1).!

comment: aString with: arg1 with: arg2 with: arg3 with: arg4 with: arg5
	"Substitute all %1...%4 with the arguments passed."

	self 
		comment: aString 
		withArguments: (
			(Array new: 5)
				at: 1 put: arg1;
				at: 2 put: arg2;
				at: 3 put: arg3;
				at: 4 put: arg4;
 				at: 5 put: arg5;
				yourself
		).!

comment: aString with: arg1 with: arg2
	"Substitute all %1...%4 with the arguments passed."

	self 
		comment: aString 
		withArguments: (Array with: arg1 with: arg2).!

comment: aString
	"Comments are the lowest priority of all messages. They may
	 be displayed in a window status area, or may be written to a
	 log. They never interrupt processing. Write aString after starting
	 a new text line."

	self 
		comment: aString 
		withArguments: #().! !

!Symbol methodsFor: 'CstMessengerBaseExt'!

cstArgumentPrintOn: aStream
	"The receiver was passed as an argument to be printed
	 into a string handled by a messenger. This method
	 determines how that argument will print itself."

	"Symbol is a subclass of String in some dialects (VW), but not all (VA)."

	^aStream nextPutAll: self! !

!CstMessageDispatcher methodsFor: 'commenting'!

commentAdd: aString
	"Comments are the lowest priority of all messages. They may
	 be displayed in a window status area, or may be written to a
	 log. They never interrupt processing. Write aString to the same
	 text line."

	self 
		commentAdd: aString 
		withArguments: #().!

commentAdd: aString with: arg1 with: arg2
	"Substitute all %1...%4 with the arguments passed."

	self 
		commentAdd: aString 
		withArguments: (Array with: arg1 with: arg2).! !

!CstMessageDispatcher methodsFor: 'confirmation'!

confirm: aString
	"Prompt the user in some way for a boolean response when asked aString."

	^self 
		confirm: aString
		withArguments: #().!

confirm: aString with: arg1 with: arg2
	"Prompt the user in some way for a boolean response when asked aString.
	 Substitute all %1...%4 with the arguments passed."

	^self 
		confirm: aString
		withArguments: (Array with: arg1 with: arg2).!

confirm: aString with: arg1 with: arg2 with: arg3
	"Prompt the user in some way for a boolean response when asked aString.
	 Substitute all %1...%4 with the arguments passed."

	^self 
		confirm: aString
		withArguments: (Array with: arg1 with: arg2 with: arg3).! !

!CstMessageDispatcher methodsFor: 'commenting'!

basicCommentAdd: aString
	"Comments are the lowest priority of all messages. They may
	 be displayed in a window status area, or may be written to a
	 log. They never interrupt processing."

	^self handler reportCommentAdd: aString.
! !

!CstMessageDispatcher methodsFor: 'confirmation'!

basicConfirmOrCancel: aString
	"Prompt the user in some way for a boolean response when asked aString."

	^self handler confirmOrCancel: aString.! !

!CstMessageDispatcher methodsFor: 'notification'!

basicNotify: aString
	"Report to the user that a non-error condition has occurred and
	 *usually* suspend processing until the user acknowledges the message."

	^self handler notify: aString.! !

!CstMessageDispatcher methodsFor: 'commenting'!

basicTranscript
	"Answer the stream that will receive some messages.
	 MessageDispatchers normaly declare what will be
	 the output stream--not the handlers."

	^Transcript! !

!CstMessageDispatcher methodsFor: 'warning'!

basicWarn: aString
	"This is an important comment that is usually given to the user
	 for a response, but could also simply be reported. It indicates
	 that something undesireable has happened or could happen."

	^self handler reportWarning: aString! !

!CstMessageDispatcher methodsFor: 'choosing'!

choose: promptString fromList: list values: values lines: lines cancel: cancelBlock

	^self handler
		choose: promptString 
		fromList: list 
		values: values 
		lines: lines 
		cancel: cancelBlock! !

!CstMessageDispatcher methodsFor: 'commenting'!

comment: aString with: arg1
	"Substitute all %1...%4 with the arguments passed."

	self 
		comment: aString 
		withArguments: (Array with: arg1).!

comment: aString with: arg1 with: arg2 with: arg3
	"Substitute all %1...%4 with the arguments passed."

	self 
		comment: aString 
		withArguments: (Array with: arg1 with: arg2 with: arg3).!

comment: aString with: arg1 with: arg2 with: arg3 with: arg4
	"Substitute all %1...%4 with the arguments passed."

	self 
		comment: aString 
		withArguments: (Array with: arg1 with: arg2 with: arg3 with: arg4).!

comment: aString withArguments: args
	"Comments are the lowest priority of all messages. They may
	 be displayed in a window status area, or may be written to a
	 log. They never interrupt processing. Write aString after starting
	 a new text line."

	| string |
	string := self print: aString withArguments: args.
	self basicComment: string.!

commentAdd: aString with: arg1 with: arg2 with: arg3
	"Substitute all %1...%4 with the arguments passed."

	self 
		commentAdd: aString 
		withArguments: (Array with: arg1 with: arg2 with: arg3).!

commentAdd: aString with: arg1 with: arg2 with: arg3 with: arg4
	"Substitute all %1...%4 with the arguments passed."

	self 
		commentAdd: aString 
		withArguments: (Array with: arg1 with: arg2 with: arg3 with: arg4).!

commentAdd: aString withArguments: args
	"Comments are the lowest priority of all messages. They may
	 be displayed in a window status area, or may be written to a
	 log. They never interrupt processing. Write aString after starting
	 a new text line."

	| string |
	string := self print: aString withArguments: args.
	self basicCommentAdd: string.! !

!CstMessageDispatcher methodsFor: 'confirmation'!

confirm: aString with: arg1
	"Prompt the user in some way for a boolean response when asked aString.
	 Substitute all %1...%4 with the arguments passed."

	^self 
		confirm: aString
		withArguments: (Array with: arg1).! !

!CstMessageDispatcher methodsFor: 'error reporting'!

error: aString with: arg1 with: arg2 with: arg3 with: arg4
	"Substitute all %1...%4 with the arguments passed."

	^self
		error: aString 
		withArguments: (Array with: arg1 with: arg2 with: arg3 with: arg4).! !

!CstMessageDispatcher methodsFor: 'confirmation'!

confirmOrCancel: aString withArguments: args
	"Prompt the user in some way for a boolean response when asked aString."

	| string |
	string := self print: aString withArguments: args.
	^self basicConfirmOrCancel: string.! !

!CstMessageDispatcher methodsFor: 'printing'!

convertString: aString
	"This allows for any transformations of aString before being displayed
	 to the user. For now, the only action is to convert backslash characters
	 to CRs. At this point, aString could also be translated into another language."

	"NLS code could go here"
	
	^self handler convertString: aString.
! !

!CstMessageDispatcher methodsFor: 'confirmation'!

confirm: aString with: arg1 with: arg2 with: arg3 with: arg4
	"Prompt the user in some way for a boolean response when asked aString.
	 Substitute all %1...%4 with the arguments passed."

	^self 
		confirm: aString
		withArguments: (Array with: arg1 with: arg2 with: arg3 with: arg4).!

confirm: aString withArguments: args
	"Prompt the user in some way for a boolean response when asked aString."

	| string |
	string := self print: aString withArguments: args.
	^self basicConfirm: string.!

confirmOrCancel: aString
	"Prompt the user in some way for a boolean response when asked aString."

	^self 
		confirmOrCancel: aString
		withArguments: #().!

confirmOrCancel: aString with: arg1
	"Prompt the user in some way for a boolean response when asked aString.
	 Substitute all %1...%4 with the arguments passed."

	^self 
		confirmOrCancel: aString
		withArguments: (Array with: arg1).!

confirmOrCancel: aString with: arg1 with: arg2
	"Prompt the user in some way for a boolean response when asked aString.
	 Substitute all %1...%4 with the arguments passed."

	^self 
		confirmOrCancel: aString
		withArguments: (Array with: arg1 with: arg2).! !

!Object methodsFor: 'CstMessengerBaseExt'!

cstArgumentPrintOn: aStream
	"The receiver was passed as an argument to be printed
	 into a string handled by a messenger. This method
	 determines how that argument will print itself."

	^self printOn: aStream! !

!CstMessageDispatcher methodsFor: 'confirmation'!

confirmOrCancel: aString with: arg1 with: arg2 with: arg3
	"Prompt the user in some way for a boolean response when asked aString.
	 Substitute all %1...%4 with the arguments passed."

	^self 
		confirmOrCancel: aString
		withArguments: (Array with: arg1 with: arg2 with: arg3).!

confirmOrCancel: aString with: arg1 with: arg2 with: arg3 with: arg4
	"Prompt the user in some way for a boolean response when asked aString.
	 Substitute all %1...%4 with the arguments passed."

	^self 
		confirmOrCancel: aString
		withArguments: (Array with: arg1 with: arg2 with: arg3 with: arg4).! !

!CstMessageDispatcher methodsFor: 'error reporting'!

error: aString
	"Raise a generic GemStone error signal with aString for the notifier string."

	^self
		error: aString 
		withArguments: #().!

error: aString with: arg1
	"Substitute all %1...%4 with the arguments passed."

	^self
		error: aString 
		withArguments: (Array with: arg1).!

error: aString with: arg1 with: arg2
	"Substitute all %1...%4 with the arguments passed."

	^self
		error: aString 
		withArguments: (Array with: arg1 with: arg2).!

error: aString with: arg1 with: arg2 with: arg3
	"Substitute all %1...%4 with the arguments passed."

	^self
		error: aString 
		withArguments: (Array with: arg1 with: arg2 with: arg3).!

error: aString withArguments: args
	"Raise an error signal with aString for the notifier string."

	| string |
	string := self print: aString withArguments: args.
	self basicError: string.! !

!CstMessageDispatcher methodsFor: 'failure reporting'!

failure: aString
	"An assertion test has failed. This is considered a low priority type of error
	 condition and execution is normally allowed to continue unless debugging."

	self reportFailures ifFalse: [^self].
	self
		failure: aString 
		withArguments: #()!

failure: aString with: arg1
	"An assertion test has failed. This is considered a low priority type of error
	 condition and execution is normally allowed to continue unless debugging.
	 Substitute all %1...%4 with the arguments passed."

	self reportFailures ifFalse: [^self].
	self
		failure: aString 
		withArguments: (Array with: arg1).!

failure: aString with: arg1 with: arg2
	"An assertion test has failed. This is considered a low priority type of error
	 condition and execution is normally allowed to continue unless debugging.
	 Substitute all %1...%4 with the arguments passed."

	self reportFailures ifFalse: [^self].
	self
		failure: aString 
		withArguments: (Array with: arg1 with: arg2).!

failure: aString with: arg1 with: arg2 with: arg3
	"An assertion test has failed. This is considered a low priority type of error
	 condition and execution is normally allowed to continue unless debugging.
	 Substitute all %1...%4 with the arguments passed."

	self reportFailures ifFalse: [^self].
	self
		failure: aString 
		withArguments: (Array with: arg1 with: arg2 with: arg3).!

failure: aString with: arg1 with: arg2 with: arg3 with: arg4
	"An assertion test has failed. This is considered a low priority type of error
	 condition and execution is normally allowed to continue unless debugging.
	 Substitute all %1...%4 with the arguments passed."

	self reportFailures ifFalse: [^self].
	self
		failure: aString 
		withArguments: (Array with: arg1 with: arg2 with: arg3 with: arg4).!

failure: aString withArguments: args
	"An assertion test has failed. This is considered a low priority type of error
	 condition and execution is normally allowed to continue unless debugging."

	| string |
	self reportFailures ifFalse: [^self].
	string := self print: aString withArguments: args.
	self basicFailure: string.! !

!CstMessageDispatcher methodsFor: 'message handling'!

handler
	"Answer an object that responds to private messages sent
	 by the receiver. "

	^handler!

handler: aMessageHandler
	"Set an object that responds to private messages sent
	 by the receiver."

	handler := aMessageHandler!

initialize

	handler := self newDefaultHandler.
!

newDefaultHandler
	"This will create an instance of a message
	 handler that is most appropriate for the
	 current environment."

	^CstMessageHandler newOwner: self! !

!CstMessageDispatcher methodsFor: 'notification'!

notify: aString
	"Report to the user that a non-error condition has occurred and
	 *usually* suspend processing until the user acknowledges the message."

	^self
		notify: aString 
		withArguments: #().!

notify: aString with: arg1
	"Report to the user that a non-error condition has occurred and
	 *usually* suspend processing until the user acknowledges the message.
	 Substitute all %1...%4 with the arguments passed."

	^self
		notify: aString 
		withArguments: (Array with: arg1).!

notify: aString with: arg1 with: arg2
	"Report to the user that a non-error condition has occurred and
	 *usually* suspend processing until the user acknowledges the message.
	 Substitute all %1...%4 with the arguments passed."

	^self
		notify: aString 
		withArguments: (Array with: arg1 with: arg2).!

notify: aString with: arg1 with: arg2 with: arg3
	"Report to the user that a non-error condition has occurred and
	 *usually* suspend processing until the user acknowledges the message.
	 Substitute all %1...%4 with the arguments passed."

	^self
		notify: aString 
		withArguments: (Array with: arg1 with: arg2 with: arg3).!

notify: aString with: arg1 with: arg2 with: arg3 with: arg4
	"Report to the user that a non-error condition has occurred and
	 *usually* suspend processing until the user acknowledges the message.
	 Substitute all %1...%4 with the arguments passed."

	^self
		notify: aString 
		withArguments: (Array with: arg1 with: arg2 with: arg3 with: arg4).!

notify: aString withArguments: args
	"Report to the user that a non-error condition has occurred and
	 *usually* suspend processing until the user acknowledges the message."

	| string |
	string := self print: aString withArguments: args.
	^self basicNotify: string.! !

!CstMessageDispatcher methodsFor: 'printing'!

print: aString
	"Substitute all %1...%4 with the arguments passed."

	^self print: aString withArguments: #()!

print: aString on: wStream 
	"Substitute all %1...%4 with the arguments passed."

	^self print: aString on: wStream withArguments: #()!

print: aString on: wStream with: arg1
	"Substitute all %1...%4 with the arguments passed."

	^self print: aString on: wStream withArguments: (Array with: arg1)!

print: aString on: wStream with: arg1 with: arg2
	"Substitute all %1...%4 with the arguments passed."

	^self print: aString on: wStream withArguments: (Array with: arg1 with: arg2)!

print: aString on: wStream with: arg1 with: arg2 with: arg3
	"Substitute all %1...%4 with the arguments passed."

	^self print: aString on: wStream withArguments: (Array with: arg1 with: arg2 with: arg3)!

print: aString on: wStream with: arg1 with: arg2 with: arg3 with: arg4
	"Substitute all %1...%4 with the arguments passed."

	^self print: aString on: wStream withArguments: (Array with: arg1 with: arg2 with: arg3 with: arg4)!

print: originalString on: wStream withArguments: anArrayOfArguments
	"Answer a copy of aString with replacement of %1...%9
	 with the print representation of an items in
	 anArrayOfArguments. Ignores $% at end of aString."

	"
		| stream |
		self new
			print: '%1 is a %2 season in the %3 hemisphere.'
			on: (stream := WriteStream on: String new)
			withArguments: #(Summer 'hot' Northern).
		^stream contents
	"

	| rStream aString |
	aString := self convertString: originalString.
	anArrayOfArguments isEmpty ifTrue: [wStream nextPutAll: aString. ^self].
	rStream := ReadStream on: aString.
	[wStream nextPutAll: (rStream upTo: $%). rStream atEnd] whileFalse: [
		rStream peek isDigit
		ifFalse: [wStream nextPut: $%]
		ifTrue: [
			| anArg |
			anArg := anArrayOfArguments at: rStream next digitValue.
			self printArgument: anArg on: wStream.
		].
	].
	"	^self	"!

print: aString with: arg1
	"Substitute all %1...%4 with the arguments passed."

	^self print: aString withArguments: (Array with: arg1)!

print: aString with: arg1 with: arg2
	"Substitute all %1...%4 with the arguments passed."

	^self print: aString withArguments: (Array with: arg1 with: arg2)!

print: aString with: arg1 with: arg2 with: arg3
	"Substitute all %1...%4 with the arguments passed."

	^self print: aString withArguments: (Array with: arg1 with: arg2 with: arg3)!

print: aString with: arg1 with: arg2 with: arg3 with: arg4
	"Substitute all %1...%4 with the arguments passed."

	^self print: aString withArguments: (Array with: arg1 with: arg2 with: arg3 with: arg4)!

print: aString withArguments: anArrayOfArguments
	"Answer a copy of aString with replacement of %1...%9
	 with the print representation of an items in
	 anArrayOfArguments. Ignores $% at end of aString."

	"	self new
			print: '%1 is a %2 month for %3.'
			withArguments: #(August 'warm' Canada).
	"

	| wStream |
	anArrayOfArguments isEmpty ifTrue: [^self convertString: aString].
	wStream := WriteStream on: (aString species new: aString size).
	self
		print: aString
		on: wStream
		withArguments: anArrayOfArguments.
	^wStream contents!

printArgument: anArg on: wStream

	self handler 
		printArgument: anArg 
		on: wStream.
! !

!CstMessageDispatcher methodsFor: 'failure reporting'!

reportFailures
	"Answer boolean whether there is an interest in reporting
	 failures."

	^self handler reportFailures 
! !

!CstMessageDispatcher methodsFor: 'request string'!

request: aString
	"Typically presents an entry field to user to allow a string to be entered."

	^self 
		request: aString 
		withArguments: #()
		default: ''.!

request: aString default: defaultString
	"Typically presents an entry field to user to allow a string to be entered."

	^self
		request: aString 
		withArguments: #()
		default: defaultString.!

request: aString with: arg1 default: defaultString
	"Prompt the user to enter a string or confirm the defaultString.
	 This normally suspends processing until an answer is given.
	 Substitute all %1...%4 with the arguments passed."

	^self
		request: aString 
		withArguments: (Array with: arg1)
		default: defaultString!

request: aString with: arg1 with: arg2 default: defaultString
	"Prompt the user to enter a string or confirm the defaultString.
	 This normally suspends processing until an answer is given.
	 Substitute all %1...%4 with the arguments passed."

	^self
		request: aString 
		withArguments: (Array with: arg1 with: arg2) 
		default: defaultString!

request: aString with: arg1 with: arg2 with: arg3 default: defaultString
	"Prompt the user to enter a string or confirm the defaultString.
	 This normally suspends processing until an answer is given.
	 Substitute all %1...%4 with the arguments passed."

	^self
		request: aString 
		withArguments: (Array with: arg1 with: arg2 with: arg3) 
		default: defaultString!

request: aString with: arg1 with: arg2 with: arg3 with: arg4 default: defaultString
	"Prompt the user to enter a string or confirm the defaultString.
	 This normally suspends processing until an answer is given.
	 Substitute all %1...%4 with the arguments passed."

	^self
		request: aString 
		withArguments: (Array with: arg1 with: arg2 with: arg3 with: arg4) 
		default: defaultString!

request: aString withArguments: args default: defaultString
	"Prompt the user to enter a string or confirm the defaultString.
	 This normally suspends processing until an answer is given.
	 Substitute all %1...%4 with the arguments passed."

	| string |
	string := self print: aString withArguments: args.
	^self 
		basicRequest: string 
		default: defaultString! !

!CstMessageDispatcher methodsFor: 'choosing'!

requestFileName: prompt default: default version: version ifFail: failBlock

	"
		self new
			requestFileName: 'Filename for example?'
			default: 'testfile.txt'
			version: #new
			ifFail: [^false].
	"

	^self handler
		requestFileName: prompt 
		default: default 
		version: version 
		ifFail: failBlock
! !

!CstMessageDispatcher methodsFor: 'commenting'!

transcript
	"Answer the stream that will receive some messages.
	 MessageDispatchers normaly declare what will be
	 the output stream--not the handlers, but this
	 double-dispatch will allow handlers to override."

	^self handler transcript! !

!CstMessageDispatcher methodsFor: 'warning'!

warn: aString
	"This is an important comment that is usually given to the user
	 for a response, but could also simply be reported. It indicates
	 that something undesireable has happened or could happen."

	^self 
		warn: aString
		withArguments: #().!

warn: aString with: arg1
	"This is an important comment that is usually given to the user
	 for a response, but could also simply be reported. It indicates
	 that something undesireable has happened or could happen.
	 Substitute all %1...%4 with the arguments passed."

	^self 
		warn: aString
		withArguments: (Array with: arg1).!

warn: aString with: arg1 with: arg2
	"This is an important comment that is usually given to the user
	 for a response, but could also simply be reported. It indicates
	 that something undesireable has happened or could happen.
	 Substitute all %1...%4 with the arguments passed."

	^self 
		warn: aString
		withArguments: (Array with: arg1 with: arg2).!

warn: aString with: arg1 with: arg2 with: arg3
	"This is an important comment that is usually given to the user
	 for a response, but could also simply be reported. It indicates
	 that something undesireable has happened or could happen.
	 Substitute all %1...%4 with the arguments passed."

	^self 
		warn: aString
		withArguments: (Array with: arg1 with: arg2 with: arg3).!

warn: aString with: arg1 with: arg2 with: arg3 with: arg4
	"This is an important comment that is usually given to the user
	 for a response, but could also simply be reported. It indicates
	 that something undesireable has happened or could happen.
	 Substitute all %1...%4 with the arguments passed."

	^self 
		warn: aString
		withArguments: (Array with: arg1 with: arg2 with: arg3 with: arg4).!

warn: aString withArguments: args
	"This is an important comment that is usually given to the user
	 for a response, but could also simply be reported. It indicates
	 that something undesireable has happened or could happen."

	| string |
	string := self print: aString withArguments: args.
	^self basicWarn: string.! !

!CstMessageDispatcher class methodsFor: 'documentation'!

copyright

	^'Copyright (C) 2000 GemStone Systems, Inc.  All Rights Reserved.'!

documentation
"
 This code was first written by Paul Baumann while employed with GemStone
 Systems, Inc.. GemStone donated this code to the Camp Smalltalk development
 effort both to help the Smalltalk community, and to distribute the effort to port
 and maintain the code.

 An instance of this class provides a common message interface that everyone
 can use for portable code. Example:

	displayClass: aClass is: validity for: purpose
		Transcript cr; show: (aClass printString, ' is the ', validity, ' class for ', purpose, '.').

 --or--

	displayClass: aClass is: validity for: purpose
		Transcript
			cr;
			nextPutAll: aClass printString;
			nextPutAll: ' is the ';
			nextPutAll: validity;
			nextPutAll: ' class for ';
			nextPutAll: purpose;
			show: $..

 Would instead be:

	displayClass: aClass is: validity for: purpose
		self cstMessenger
			comment: '%1 is the %2 class for %3.'
			with: aClass
			with: validity
			with: purpose.

 Some advantages are: 
	* it looks cleaner.
	* the message is a single string that can be NLS translated.
	* temporary strings aren't created only to be immediately GC'ed (#printString and #,).
	* the printing of arguments can be wrapped with exception handlers.
	* the #messenger send allows for context sensitive message handling.
	* reduces the number of messages to the stream itself.
	* the messenger can handle messages in headless images.
	* it can be used to hide differences between Smalltalk dialects.
	* possible to add a 'don't tell me again' check box to dialogs.
	* no direct reference to the output class or global (better style for maintainability).
	* parameters can appear more than once in a string. 

 Some protocols currently supported are:

	comment:	(ex: Write to transcript after cr)
	commentAdd:	(ex: Write to transcript without cr)
	confirm:	(ex: Open a Yes/No dialog)
	confirmOrCancel:	(ex: Open a Yes/No/Cancel dialog)
	error:	(ex: Raise a generic error--but with argument substitution)
	failure:	(ex: An assertion test has failed--only of interest to developers)
	notify:	(ex: Open a message dialog box)
	print:	(ex: Return a string with translation and argument substitution)
	print:on:	(same as #print: but to a stream)
	request:default:	(ex: Open a dialog that returns an input string)
	warn:		(ex: Open a warning dialog)

 The above examples are really only examples because what will really happen
 depends on the dialect being used, and the environment. The user might be
 on a remote machine. The messages above each support at least four
 arguments; for example:

	comment:
	comment:with:
	comment:with:with:
	comment:with:with:with:
	comment:with:with:with:with:
	comment:withArguments:

"! !

!CstMessageDispatcher class methodsFor: 'class initialization'!

initialize
	"The singleton and Handler is lazy-initialized, so
	 it is OK to simply nil the variables to release
	 any current settings."

	"	self initialize.	"

	self allSubclasses do: [:ea | ea singleton: nil ].
	self singleton: nil.
! !

!CstMessageDispatcher class methodsFor: 'documentation'!

license

^'The contents of this file are subject to the Mozilla Public
License Version 1.1 (the "License"); you may not use this file
except in compliance with the License. You may obtain a copy of
the License at http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS
IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
implied. See the License for the specific language governing
rights and limitations under the License.

The Original Code is Messenger Support.

The Initial Developer of the Original Code is GemStone Systems, Inc.
Portions created by GemStone Systems, Inc. are
Copyright (C) 2000 GemStone Systems, Inc.  All
Rights Reserved.

Contributor(s):
	Paul Baumann <pbaumann@gemstone.com>
'! !

!CstMessageDispatcher class methodsFor: 'class initialization'!

new
	^super new initialize
! !

!CstMessageDispatcher class methodsFor: 'accessors'!

singleton
	"Note that singleton is a class instance variable instead
	 of just a class variable."

		"	self singleton.	"

	singleton isNil ifTrue: [self singleton: self new].
	^singleton!

singleton: aDispatcher
	
	singleton := aDispatcher! !

!CstMessageHandler methodsFor: 'private'!

choose: messageString fromList: list values: listValues lines: maxLines cancel: cancelBlock 

	^cancelBlock value.
!

choose: multilineMessage labels: labelArray values: returnValueArray default: defaultValue

	^defaultValue!

confirm: aString
	"Prompt the user in some way for a boolean response when asked aString."

	^true!

confirmOrCancel: aString
	"Prompt the user in some way for a boolean response when asked aString."

	^true!

convertString: aString
	"This allows for any transformations of aString before being displayed
	 to the user. For now, the only action is to convert backslash characters
	 to CRs. At this point, aString could also be translated into another language."

	| string |
	string := self translate: aString.
	(string includes: $\) ifFalse: [^string].
	"This was the only portable way I knew
	 to find out what the line delimiter is."
	^string 
		copyReplaceAll: '\' 
		with: ((WriteStream on: String new) cr; contents).
!

initialize
	"Do nothing by default."!

notify: aString
	"Report to the user that a non-error condition has occurred and
	 *usually* suspend processing until the user acknowledges the message."

	^self reportComment: (
		self cstMessenger
			print: 'Notice: %1' with: aString
	 )!

owner
	"Answer an instance of a MessageDispatcher that is my owner."
	^owner!

owner: aMessageDispatcher
	owner := aMessageDispatcher!

printArgument: anArg on: wStream
	"It would be nice to wrap this in an exception handler
	 so that an error message will print if there is a
	 problem while printing."

	anArg cstArgumentPrintOn: wStream.
!

reportComment: aString
	"Comments are the lowest priority of all messages. They may
	 be displayed in a window status area, or may be written to a
	 log. They never interrupt processing."

	self transcript cr.
	self reportCommentAdd: aString.
	^aString!

reportCommentAdd: aString
	"Comments are the lowest priority of all messages. They may
	 be displayed in a window status area, or may be written to a
	 log. They never interrupt processing."

	self transcript show: aString.
	^aString!

reportError: aString
	"Raise an error signal with aString for the notifier string."

	^self error: aString.!

reportFailure: errorString
	"An assertion test has failed. This is considered a low priority type of error
	 condition and execution is normally allowed to continue unless debugging."

	^self reportError: (
		self cstMessenger
			print: 'Failure-%1' 
			with: errorString
	 ).!

reportFailures
	"Answer boolean whether there is an interest in reporting
	 failures."

	^true 
!

reportWarning: aString
	"This is an important comment that is usually given to the user
	 for a response, but could also simply be reported. It indicates
	 that something undesireable has happened or could happen."

	^self notify: (
		self cstMessenger
			print: 'Warning: %1'
			with: aString
	 ).!

request: aString default: defaultString
	"Prompt the user to enter a string or confirm the defaultString.
	 This normally suspends processing until an answer is given."

	^defaultString!

requestFileName: prompt default: default version: version ifFail: failBlock

	^(default isNil or: [default isEmpty])
		ifTrue: [failBlock value]
		ifFalse: [default].

!

transcript
	"Answer the stream that will receive some messages.
	 MessageDispatchers normaly declare what will be
	 the output stream--not the handlers, but this
	 double-dispatch will allow handlers to override."

	self owner isNil ifTrue: [^Transcript].
	^self owner basicTranscript.!

translate: aString
	"This allows for any transformations of aString before being displayed
	 to the user. At this point, aString could be translated into another language."

	^aString
! !

!CstMessageHandler class methodsFor: 'documentation'!

documentation
"
 Instances of this class (or subclasses) will have messages routed to it
 by instances of CstMessageDispatcher. This class contains behavior
 that may be considered specific to a particular Smalltalk dialect or
 or even the packaging of a particular Smalltalk dialect.

 The class CstMessageHandler contains the most generic behavior
 possible. Any code there should give a reasonable response in a
 headless image for any Smalltalk dialect. However, that code usually
 isn't best for any given image, so a subclass is usually used.

 Multiple subclasses of CstMessageHandler can co-exist in an image.
 The correct subclass for an image is selected using the class
 method #mightApplyForImage. No subclass of CstMessageHandler
 should have a direct reference to a class or global that may not exist
 in any other type of image that the code may be loaded into.

 Refer to the 'Bridge' pattern of the GOF 'Design Patterns' book for a better
 idea of the purpose of this class.
"! !

!CstMessageHandler class methodsFor: 'As yet unclassified'!

mightApplyForImage
	"Answer boolean whether instances of this class
	 should be used for the type of image currently
	 being used. The default is false so that subclasses
	 have higher priority in the selection. This
	 method should be overridden in subclasses."

	^false
! !

!CstMessageHandler class methodsFor: 'Not categorized'!

new
	^super new initialize!

newOwner: aMessageDispatcher
	"Answer an instance of the most appropriate kind of
	 CstMessageHandler to use for the image in use."

	^self selectCorrectBehavior new
		owner: aMessageDispatcher! !

!CstMessageHandler class methodsFor: 'As yet unclassified'!

selectCorrectBehavior
	"Select the most appropriate kind of
	 CstMessageHandler to use for the image in use."

	| candidates |
	candidates := self withAllSubclasses asArray		"Squeak starts with Set"
		select: [:ea | ea mightApplyForImage ].
	candidates isEmpty
		ifTrue: [ ^self ].
	candidates size = 1
		ifTrue: [ ^candidates first ].
	^candidates last.! !

!CstMessageHandlerVa methodsFor: 'user interface'!

choose: promptString fromList: list values: values lines: lines cancel: cancelBlock
	" If the CommonWidgets classes aren't present (probably because
	 we're in a headless runtime), just return the value of cancelBlock."

	| prompter result cwListPrompter |
	cwListPrompter := Smalltalk 
		at: #CwListPrompter 
		ifAbsent: [ ^ cancelBlock value ].
	prompter := cwListPrompter new
		parent: nil;
		messageString: promptString;
		selectedItems: #();
		items: values;
		printBlock: [ :e | list at: (values indexOf: e) ];
		returnIndex: true.
	result := prompter prompt.
	^result isNil
		ifTrue: [ cancelBlock value ]
		ifFalse: [ values at: result ]!

choose: multilineMessage labels: labelArray values: returnValueArray default: defaultValue
	"Open a prompter to display the given message and a line of buttons.  Return the value
	 of the selected button, or the default value if the dialog is terminated by the user.
	 If the CommonWidgets classes aren't present (probably because we're in a headless runtime),
	   just return the default value."

	| shell box button label lastButton valueStream motif cwWMShell cwAppContext|
	"Bind to the CW classes and constants"
	lastButton := nil.
	motif :=  Smalltalk
		poolDictionaryAt: #CwConstants
		ifAbsent: [^defaultValue].
	cwWMShell := Smalltalk
		at: #CwWMShell
		ifAbsent: [^defaultValue].
	cwAppContext := Smalltalk
		at: #CwAppContext
		ifAbsent: [^defaultValue].
	shell :=  (cwWMShell focusShell) createDialogShell:  'PrompterShell' argBlock: [:w | ].
	box := shell createForm: 'Form' argBlock: [:w |
			w width: 300; height: 60.
			w dialogTitle: 'Response Required';
			dialogStyle: (motif at: 'XmDIALOGFULLAPPLICATIONMODAL') ].
	label := box createLabel: 'Prompt' argBlock: [:l |
		l alignment: (motif at: 'XmALIGNMENTBEGINNING');
			leftAttachment: (motif at: 'XmATTACHFORM'); leftOffset: 5;
			rightAttachment: (motif at: 'XmATTACHFORM'); rightOffset: 5;
			topAttachment: (motif at: 'XmATTACHFORM');
			labelString: multilineMessage
		].
	label manageChild.
	valueStream := returnValueArray readStream.
	labelArray do: [:string |
		button := box createPushButton: 'btn' , (labelArray indexOf: string) printString argBlock: [:w |
			w topAttachment: (motif at: 'XmATTACHWIDGET'); topWidget: label; topOffset: 20;
				leftAttachment: (lastButton isNil ifTrue: [ (motif at: 'XmATTACHFORM') ] ifFalse: [ (motif at: 'XmATTACHWIDGET') ]);
				leftWidget: lastButton; leftOffset: 20.
			w labelString: string;
				recomputeSize: true.
			].
		button manageChild.
		lastButton := button.
		valueStream peek = defaultValue ifTrue: [ box defaultButton: button ].
		button addCallback: (motif at: 'XmNactivateCallback') receiver: self selector: #prompterButtonCallback:clientData:callData:
				clientData: (Array with: box with: valueStream next).
		].
	box deferRedraw: [ box setInitialSize. box width: label width + label x + label x; height: button y + button height + 10 ].
	box manageChild.
	cwAppContext default readAndDispatchWhile: [
		box isDestroyed not and: [ box isManaged ]	].
	^box userData isNil
		ifTrue: [ defaultValue ]
		ifFalse: [box userData]! !

!CstMessageHandlerVa methodsFor: 'As yet unclassified'!

confirm: aString
	"Prompt the user in some way for a boolean response when asked aString."

	| prompter |
	(prompter := self prompter) isNil 
		ifTrue: [^super confirm: aString].
	^prompter confirm: aString.!

confirmOrCancel: aString
	"Prompt the user in some way for a boolean response when asked aString."

	| prompter |
	(prompter := self prompter) isNil 
		ifTrue: [^super confirmOrCancel: aString].
	^prompter confirmYesNoCancel: aString.!

convertString: aString
	"This allows for any transformations of aString before being displayed
	 to the user. For now, the only action is to convert backslash characters
	 to CRs. At this point, aString could also be translated into another language."

	| string |
	string := self translate: aString.
	(string includes: $\) ifFalse: [^string].
	^string addLineDelimiters.! !

!CstMessageHandlerVa methodsFor: 'Not categorized'!

hostErrorSignal

	^hostSignal! !

!CstMessageHandlerVa methodsFor: 'As yet unclassified'!

initialize
	"Cache objects that may need to be referenced frequently
	 by a name that isn't portable between dialects."

	hostSignal := (Smalltalk at: #SystemExceptions) at: 'ExAll'.
	hostPrompter := Smalltalk at: #CwMessagePrompter ifAbsent: [].
!

notify: aString
	"Report to the user that a non-error condition has occurred and
	 *usually* suspend processing until the user acknowledges the message."

	| prompter |
	(prompter := self prompter) isNil 
		ifTrue: [^super notify: aString].
	^prompter message: aString.!

printArgument: anArg on: wStream

	[anArg cstArgumentPrintOn: wStream]
		when: self hostErrorSignal
		do: [:ex | 
			wStream nextPutAll: '<#printOn: Error>'. 
			ex exitWith: nil.
		]
!

prompter

	^hostPrompter
!

request: message default: initialAnswer
	"Prompt the user to enter a string or confirm the initialAnswer.
	 This normally suspends processing until an answer is given. If
	 the CommonWidgets classes aren't present (probably because
	 we're in a headless runtime), just return the initial answer."

	| prompter cwExtendedTextPrompter cwAppContext |
	cwExtendedTextPrompter := Smalltalk at: #CwExtendedTextPrompter ifAbsent: [^initialAnswer].
	cwAppContext := Smalltalk at: #CwAppContext ifAbsent: [^initialAnswer].
	prompter := cwExtendedTextPrompter new
		messageString: message;
		result: initialAnswer.
	prompter createWindow.
	prompter openPrompter.

	"Required on some platforms to make prompter appear."
	prompter shell updateWidget.
	
	prompter textWidget value: initialAnswer; setSelection: 0@initialAnswer size.

	cwAppContext default 
		readAndDispatchWhile: [prompter shell isDestroyed not and: [prompter done not]].
	prompter shell isDestroyed 
		ifTrue: [prompter setDestroyedResult].
	prompter destroyPrompter.
	prompter parent isDestroyed
		ifFalse: [prompter parent updateDisplay].
	^prompter result! !

!CstMessageHandlerVa methodsFor: 'user interface'!

requestFileName: prompt default: default version: version ifFail: failBlock
	" If the CommonWidgets classes aren't present (probably because we're in
	 a headless runtime), return the result of evaluating failBlock."

	| fname cwFileSelectionPrompter |
	cwFileSelectionPrompter := Smalltalk 
		at: #CwFileSelectionPrompter 
		ifAbsent: [^failBlock value ].
	fname := (cwFileSelectionPrompter new title: prompt;
		searchMask: '*.*'; fileName: default) prompt.
	(version = #new and: [((Smalltalk at: #CfsStat) stat: fname) isCfsError not]) ifTrue: [
		(self cstMessenger confirm: '%1 exists - use anyway?' with: fname) 
		ifFalse: [ ^failBlock value ].
	].
	(version = #old and: [((Smalltalk at: #CfsStat) stat: fname) isCfsError not]) ifTrue: [
		(self cstMessenger confirm: '%1 doesn'' exist - try again?' with: fname)
		ifTrue: [ ^self requestFileName: prompt default: default version: version ifFail: failBlock ]
		ifFalse: [ ^failBlock value ].
	].
	(fname isNil or: [ fname size = 0 ])
 		ifTrue: [ ^failBlock value ]
		ifFalse: [ ^fname ]! !

!String methodsFor: 'CstMessengerBaseExt'!

cstArgumentPrintOn: aStream
	"The receiver was passed as an argument to be printed
	 into a string handled by a messenger. This method
	 determines how that argument will print itself."

	^aStream nextPutAll: self! !

!CstMessageHandlerVa class methodsFor: 'As yet unclassified'!

mightApplyForImage

	| sys |
	sys := Smalltalk at: #System ifAbsent: [^false].
	^sys vmType = 'ES'

! !

!CstMessageHandlerVw methodsFor: 'multiple choice dialogs'!

choose: messageString fromList: list values: listValues lines: maxLines cancel: cancelBlock
	| prompter |
	(prompter := self prompter) isNil ifTrue: [
		^super
			choose: messageString
			fromList: list
			values: listValues
			lines: maxLines
			cancel: cancelBlock
	].
	^prompter
		choose: messageString
		fromList: list
		values: listValues
		lines: maxLines
		cancel: cancelBlock!

choose: messageString labels: labels values: values default: defaultValue
	| prompter |
	(prompter := self prompter) isNil ifTrue: [
		^super
			choose: messageString
			labels: labels
			values: values
			default: defaultValue
	].
	^prompter
		choose: messageString
		labels: labels
		values: values
		default: defaultValue! !

!CstMessageHandlerVw methodsFor: 'As yet unclassified'!

confirm: aString
	"Prompt the user in some way for a boolean response when asked aString."

	| prompter |
	(prompter := self prompter) isNil
		ifTrue: [^super confirm: aString].
	^prompter confirm: aString.!

confirmOrCancel: aString
	"Prompt the user in some way for a boolean response when asked aString."

	^self 
		choose: aString 
		labels: #('Yes' 'No' 'Cancel') 
		values: #(true false nil) 
		default: nil!

convertString: aString
	"This allows for any transformations of aString before being displayed
	 to the user. For now, the only action is to convert backslash characters
	 to CRs. At this point, aString could also be translated into another language."

	| string |
	string := self translate: aString.
	(string includes: $\) ifFalse: [^string].
	^string withCRs.! !

!CstMessageHandlerVw methodsFor: 'Not categorized'!

hostErrorSignal

	^hostSignal!

initialize
	"Cache objects that may need to be referenced frequently
	 by a name that isn't portable between dialects."

	hostSignal := Smalltalk at: #Error ifAbsent: [Object errorSignal].
	hostPrompter := (Smalltalk at: #UI ifAbsent: [Smalltalk])
		at: #Dialog ifAbsent: [].! !

!CstMessageHandlerVw methodsFor: 'As yet unclassified'!

notify: aString
	"Report to the user that a non-error condition has occurred and
	 *usually* suspend processing until the user acknowledges the message."

	| prompter |
	(prompter := self prompter) isNil
		ifTrue: [^super notify: aString].
	^prompter warn: aString.! !

!CstMessageHandlerVw methodsFor: 'Not categorized'!

printArgument: anArg on: wStream

	[anArg cstArgumentPrintOn: wStream]
		on: self hostErrorSignal
		do: [:ex |
			wStream nextPutAll: '<#printOn: Error>'.
			ex return: nil.
		]! !

!CstMessageHandlerVw methodsFor: 'file name dialogs'!

prompter

	^hostPrompter! !

!CstMessageHandlerVw methodsFor: 'As yet unclassified'!

request: message default: initialAnswer
	"Prompt the user to enter a string or confirm the initialAnswer.
	 This normally suspends processing until an answer is given. If
	 the CommonWidgets classes aren't present (probably because
	 we're in a headless runtime), just return the initial answer."

	| prompter |
	(prompter := self prompter) isNil
		ifTrue: [^super request: message default: initialAnswer].
	^prompter request: message initialAnswer: initialAnswer! !

!CstMessageHandlerVw methodsFor: 'file name dialogs'!

requestFileName: message default: default version: versionType ifFail: failBlock
	| prompter |
	(prompter := self prompter) isNil ifTrue: [
		^super
			requestFileName: message
			default: default
			version: versionType
			ifFail: failBlock
	].
	^prompter
		requestFileName: message
		default: default
		version: versionType
		ifFail: failBlock! !

!CstMessageHandlerVw class methodsFor: 'As yet unclassified'!

mightApplyForImage

	(Smalltalk respondsTo: #versionName) ifFalse: [^false].
	^'VisualWorks*' match: Smalltalk versionName
! !


"	***** Class initialization *****	"!

CstMessageDispatcher initialize.!
"	***** End of file *****	"!