Object subclass: #OperaNamedObjectHome
	instanceVariableNames: 'objectDict tempObjectCounter '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Opera-Server'!

!OperaNamedObjectHome methodsFor: 'initialize-release'!
initialize
	objectDict := IdentityDictionary new.
	tempObjectCounter := 0.! !


!OperaNamedObjectHome methodsFor: 'actions' stamp: 'MU 2/22/2002 01:12'!
at: aSymbol
	
	^ objectDict at: aSymbol ifAbsent:[OperaError noImplement signal: aSymbol]! !

!OperaNamedObjectHome methodsFor: 'actions'!
at: aSymbol ifAbsent: aBlock
	^ objectDict at: aSymbol ifAbsent: aBlock! !

!OperaNamedObjectHome methodsFor: 'actions'!
at: aSymbol ifAbsentPut: aBlock
	self checkCleanTempObjects.
	^ objectDict at: aSymbol ifAbsentPut: aBlock! !

!OperaNamedObjectHome methodsFor: 'actions'!
at: aSymbol put: anObject 
	self checkCleanTempObjects.
	^ objectDict at: aSymbol put: anObject! !

!OperaNamedObjectHome methodsFor: 'actions'!
removeKey: aSymbol
	^ objectDict removeKey: aSymbol ! !

!OperaNamedObjectHome methodsFor: 'actions'!
removeKey: aSymbol ifAbsent: aBlock
	^ objectDict removeKey: aSymbol ifAbsent: aBlock! !


!OperaNamedObjectHome methodsFor: 'factory'!
defaultDictionary
	^IdentityDictionary new! !


!OperaNamedObjectHome methodsFor: 'private' stamp: 'MU 2/28/2002 17:59'!
checkCleanTempObjects
	tempObjectCounter := tempObjectCounter + 1.
	tempObjectCounter > 999999
		ifTrue: 
			[[objectDict keysAndValuesRemove: [:key :value | key beginsWith: #temp:]] valueUninterruptably.
			tempObjectCounter := 0]! !


!OperaNamedObjectHome methodsFor: 'printing'!
printOn: aStream
	aStream nextPutAll:'objectHome:'.
	aStream cr. 
	objectDict printOn: aStream! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

OperaNamedObjectHome class
	instanceVariableNames: ''!

!OperaNamedObjectHome class methodsFor: 'instance creation'!
new
	^super new initialize! !


Object subclass: #OperaObjectAdapter
	instanceVariableNames: 'namedObjectHome oidGenerator '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Opera-Server'!

!OperaObjectAdapter methodsFor: 'initialize-release'!
initialize
	oidGenerator := nil.
	self initializeNamedObjectHome! !

!OperaObjectAdapter methodsFor: 'initialize-release'!
initializeNamedObjectHome
	
	self namedObjectHome initialize! !


!OperaObjectAdapter methodsFor: 'private' stamp: 'MU 1/22/2002 11:36'!
activateObjectReference: anObjectReference named: aRegisteredObjectName 
	| found |
	"assign object name"
	"implicitly create unshared obj ref name and register"
	found := self at: aRegisteredObjectName ifAbsent: [].
	found isNil
		ifTrue: 
			[|  impl nm | 
			impl := anObjectReference impl.
			nm := self nextTempObjectNameFor: impl.
			self at: nm put: impl. 
			anObjectReference name: nm]
		ifFalse: [anObjectReference name: aRegisteredObjectName]. 
	^ anObjectReference! !

!OperaObjectAdapter methodsFor: 'private'!
nextTempObjectName
	^('temp:', self oidGenerator nextValue) asSymbol! !

!OperaObjectAdapter methodsFor: 'private'!
nextTempObjectNameFor: anObject
	| wStr |
	"^('temp:', self oidGenerator nextValue,':', anObject name) asSymbol"
	wStr := WriteStream on: (String new: 32).
	wStr nextPutAll: 'temp:'.
	wStr nextPutAll: self oidGenerator nextValue.
	wStr nextPutAll: ':'.
	wStr nextPutAll: anObject class name.
	^wStr contents asSymbol! !


!OperaObjectAdapter methodsFor: 'tiny naming service'!
at: aSymbol
	^ self namedObjectHome at: aSymbol! !

!OperaObjectAdapter methodsFor: 'tiny naming service'!
at: aSymbol ifAbsent: aBlock
	^ self namedObjectHome at: aSymbol ifAbsent: aBlock! !

!OperaObjectAdapter methodsFor: 'tiny naming service'!
at: aSymbol ifAbsentPut: aBlock
	^ self namedObjectHome at: aSymbol ifAbsentPut: aBlock! !

!OperaObjectAdapter methodsFor: 'tiny naming service'!
at: aSymbol put: anObject 
	^ self namedObjectHome at: aSymbol put: anObject! !

!OperaObjectAdapter methodsFor: 'tiny naming service'!
removeKey: aSymbol
	^ self namedObjectHome removeKey: aSymbol ! !

!OperaObjectAdapter methodsFor: 'tiny naming service'!
removeKey: aSymbol ifAbsent: aBlock
	^ self namedObjectHome removeKey: aSymbol ifAbsent: aBlock! !


!OperaObjectAdapter methodsFor: 'receiving' stamp: 'MU 2/20/2002 01:01'!
dispatch: anOperaRemoteMessage
	
	| message receiver ret |
	message := anOperaRemoteMessage.
	receiver := self at: message receiverName.
	OperaPortableUtil iterate: message arguments
		keysAndValuesDo: [:idx :each | (OperaObjectReference hasInstance: each)
				ifTrue: [message arguments at: idx put: each asRemoteObject]].

	"##need more work##"
	"need error handling for obj ref"
	ret := receiver perform: message selector withArguments: message arguments.

	"##restrict implicit self return-by-value##"
	ret == receiver ifTrue: [^ ret := (self activateObjectReference: (OperaObjectReference on: ret )
				named: message receiverName)].

	"##investigate return-by-value or return-by-ref##"
	(OperaObjectReference hasInstance: ret)
		ifTrue: [^ ret := (self activateObjectReference: ret named: ret name)].

	^ ret! !


!OperaObjectAdapter methodsFor: 'registering'!
registerObjectImplFor: anObjectReference
	| found |
	(OperaObjectReference hasInstance: anObjectReference) ifFalse:[OperaError badParameter; description:'parameter is not object refrence'; signal].
	
	"if found, object has been registered already, so do nothing"
	
	found := self at: anObjectReference name ifAbsent: [].
	found isNil
		ifTrue: 
			[| nm impl |
			impl :=  anObjectReference impl.
			nm := self nextTempObjectNameFor: impl.
			self at: nm put: impl.
			anObjectReference name: nm].

	^ anObjectReference! !


!OperaObjectAdapter methodsFor: 'accessing' stamp: 'MU 2/13/2002 00:29'!
namedObjectHome
	namedObjectHome isNil ifTrue:[namedObjectHome := OperaNamedObjectHome new].
	^namedObjectHome! !

!OperaObjectAdapter methodsFor: 'accessing'!
namedObjectHome: aNamedObjectHome
	namedObjectHome := aNamedObjectHome
	! !

!OperaObjectAdapter methodsFor: 'accessing'!
oidGenerator
	oidGenerator isNil ifTrue:[oidGenerator := OperaOidGenerator new].
	^oidGenerator! !

!OperaObjectAdapter methodsFor: 'accessing'!
oidGenerator: aNamedObjectHome
	oidGenerator := aNamedObjectHome
	! !

!OperaObjectAdapter methodsFor: 'accessing' stamp: 'MU 2/14/2002 01:25'!
service
	^service! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

OperaObjectAdapter class
	instanceVariableNames: 'default '!

!OperaObjectAdapter class methodsFor: 'class initialization' stamp: 'MU 2/28/2002 16:21'!
initialize
	"OperaObjectAdapter initialize"
	super initialize.
	OperaPortableUtil installRootOOA! !

!OperaObjectAdapter class methodsFor: 'class initialization' stamp: 'MU 2/17/2002 15:38'!
obsolete
	OperaPortableUtil uninstallRootOOA.
	super obsolete.
	! !


!OperaObjectAdapter class methodsFor: 'constants' stamp: 'MU 2/13/2002 00:53'!
dispatchSelector
	^#dispatch:! !

!OperaObjectAdapter class methodsFor: 'constants' stamp: 'MU 2/13/2002 00:54'!
dispatchServiceName
	^'Dispatch'! !

!OperaObjectAdapter class methodsFor: 'constants' stamp: 'MU 2/13/2002 00:58'!
dispatchSoapSignature
	^(SoapServiceSignature name: self dispatchServiceName paramNames: #(aRemoteMessageSend)).! !

!OperaObjectAdapter class methodsFor: 'constants'!
serverName
	^'RootOOA'! !

!OperaObjectAdapter class methodsFor: 'constants'!
serverURIName
	^'urn:SoapOpera:', self serverName! !

!OperaObjectAdapter class methodsFor: 'constants' stamp: 'MU 2/14/2002 01:13'!
serviceTargetObjectURI
	^'/soapOpera/rootOOA'! !


!OperaObjectAdapter class methodsFor: 'instance creation' stamp: 'MU 2/28/2002 16:24'!
default
	default isNil ifTrue:[default := self new].
	^default! !

!OperaObjectAdapter class methodsFor: 'instance creation' stamp: 'MU 2/28/2002 16:25'!
new
	^super new initialize! !

!OperaObjectAdapter class methodsFor: 'instance creation' stamp: 'MU 2/28/2002 16:24'!
root
	"OperaObjectAdapter root"
	"Returns RootOOA"
	^self default! !

OperaObjectAdapter initialize!