Object subclass: #SoapAbstractConnector
	instanceVariableNames: 'envelopeBuilder '
	classVariableNames: 'ShowLog '
	poolDictionaries: ''
	category: 'SOAP-ClientBase'!
!SoapAbstractConnector commentStamp: 'MU 10/5/2002 01:36' prior: 0!
I connect to some server.
My subclasses will know what socket client should be used for connecting right server.

---
MU 10/5/2002 01:35!


!SoapAbstractConnector methodsFor: 'actions' stamp: 'MU 8/26/2002 23:42'!
debugReturn: aSoapMessage 

	self class showLog
		ifTrue: [
			| envelope context |
			envelope := aSoapMessage envelope.
			context := aSoapMessage context.
			Transcript cr; cr; show: '##'; show: super printString;
						show: ' Client:Return:{'; cr; show: envelope printString; cr; show: '}'.
			Transcript cr; show: '-> with ->'.
			Transcript cr; show: context printString]! !

!SoapAbstractConnector methodsFor: 'actions' stamp: 'mu 1/5/2002 00:22'!
debugSend: aSoapEnvelope to: anSoapLocatorObject with: aSoapContext
	
	self class showLog 
		ifTrue: [Transcript cr; show: '##', super printString, ' Client:Request:{'.
			Transcript cr; show: aSoapEnvelope printString.
			Transcript cr; show: '} to ->'.
			Transcript cr; show: anSoapLocatorObject printString.
			Transcript cr; show: '-> with ->'.
			Transcript cr; show: aSoapContext printString].
! !

!SoapAbstractConnector methodsFor: 'actions' stamp: 'mu 1/5/2002 00:08'!
send: aSoapEnvelope to: anSoapLocatorObject with: aSoapContext
	"1. add an addittinal protocolSpecific header to aSoapEnvelope"
	"2. send a SoapMessage to a remote server"
	"3. get the result and unmarshall it"
	"4. return aSoapMessage or nil"
	
	^self subclassResponsibility! !


!SoapAbstractConnector methodsFor: 'accessing' stamp: 'mu 8/28/2002 16:54'!
envelopeBuilder
	envelopeBuilder isNil ifTrue: [envelopeBuilder := SoapEnvelopeBuilder new].
	^envelopeBuilder
! !

!SoapAbstractConnector methodsFor: 'accessing' stamp: 'mu 8/28/2002 16:54'!
envelopeBuilder: aSoapEnvelopeBuilder
	envelopeBuilder := aSoapEnvelopeBuilder! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

SoapAbstractConnector class
	instanceVariableNames: ''!

!SoapAbstractConnector class methodsFor: 'class initialization' stamp: 'MU 10/5/2002 02:26'!
initialize
	"SoapAbstractConnector initialize"
	ShowLog := false.
	
	self initializeAfterLoad! !

!SoapAbstractConnector class methodsFor: 'class initialization' stamp: 'mu 5/1/2003 18:00'!
initializeAfterLoad
		
	self name == #SoapAbstractConnector
		ifFalse: [SoapConnectorMap at: self transportName asSymbol put: self].
! !

!SoapAbstractConnector class methodsFor: 'class initialization' stamp: 'mu 8/28/2002 17:06'!
initializeAll
	"SoapAbstractConnector initializeAll"
	
	self subclassesDo: [:each | each initialize]! !

!SoapAbstractConnector class methodsFor: 'class initialization' stamp: 'mu 5/1/2003 18:00'!
obsolete
	self name == #SoapAbstractConnector
		ifFalse: [SoapConnectorMap removeAt: self transportName asSymbol].
	super obsolete! !


!SoapAbstractConnector class methodsFor: 'setting' stamp: 'mu 1/5/2002 00:15'!
showLog
	^ShowLog! !

!SoapAbstractConnector class methodsFor: 'setting' stamp: 'mu 1/5/2002 00:15'!
showLog: aBoolean
	ShowLog := aBoolean! !


!SoapAbstractConnector class methodsFor: 'constants' stamp: 'mu 8/28/2002 17:02'!
transportName
	^#noName! !


Object subclass: #SoapAbstractSocketClient
	instanceVariableNames: 'socket serverAddress host port reqURI '
	classVariableNames: 'HostAddressCache UseHostAddressCache '
	poolDictionaries: ''
	category: 'SOAP-ClientBase'!
!SoapAbstractSocketClient commentStamp: 'MU 10/5/2002 01:38' prior: 0!
I use some socket to send packets to server. 
My subclasses will know what tranport protocol should be used in that socket.

---
MU 10/5/2002 01:35!


!SoapAbstractSocketClient methodsFor: 'private' stamp: 'mu 9/8/2003 00:22'!
addressForName: hostNameOrIp timeout: timeout
	| address |
	address :=  [NetNameResolver addressForName: hostNameOrIp timeout: timeout]
				on: NameLookupFailure
				do: [:ex | ex return: nil].
	^address! !

!SoapAbstractSocketClient methodsFor: 'private' stamp: 'mu 9/8/2003 00:23'!
cachedAddressFor: hostName 
	"utility for fast hostAddress lookup - if not found, just return nil"
	| timeout cache |
	hostName = 'localhost'
		ifTrue: [^ #(127 0 0 1 ) asByteArray].
	timeout := self class resolveTimeout.
	self class useHostAddressCache
		ifFalse: [^self addressForName: hostName timeout: timeout].
	cache := self class hostAddressCache.
	(cache includesKey: hostName)
		ifTrue: [^ cache at: hostName]
		ifFalse: [| resolvedAddress | 
			resolvedAddress := self addressForName: hostName timeout: timeout.
			^ resolvedAddress isNil
				ifFalse: [cache at: hostName put: resolvedAddress]]! !

!SoapAbstractSocketClient methodsFor: 'private' stamp: 'MU 9/18/2002 12:26'!
initializeNetwork
	Socket initializeNetwork.! !

!SoapAbstractSocketClient methodsFor: 'private' stamp: 'MU 4/18/2003 16:08'!
resolveServerAddress
	serverAddress := self cachedAddressFor: host.
	serverAddress isNil
		ifTrue: [self class useHostAddressCache
				ifTrue: [self class hostAddressCache
						removeKey: host
						ifAbsent: []].
			SoapCannotResolveServer signal: 'Could not resolve the server named: ' , host]! !


!SoapAbstractSocketClient methodsFor: 'actions' stamp: 'mu 9/8/2003 00:31'!
send: soapEnvelope 
	| cont |
	cont := nil.
	self prepareEnvelope: soapEnvelope.
	self prepareAdditionals.
	self initializeNetwork.
	self resolveServerAddress.
	[self prepareSocket.
	self sendBySocket. 
	cont := self getResponseContents]
		ensure: [self releaseSocket].
	^ cont! !


!SoapAbstractSocketClient methodsFor: 'actions-hooks'!
getResponseContents
	self subclassResponsibility! !

!SoapAbstractSocketClient methodsFor: 'actions-hooks'!
prepareAdditionals
	"By default - do nothing"! !

!SoapAbstractSocketClient methodsFor: 'actions-hooks'!
prepareEnvelope: soapEnvelope
	self subclassResponsibility! !

!SoapAbstractSocketClient methodsFor: 'actions-hooks'!
prepareSocket
	self subclassResponsibility! !

!SoapAbstractSocketClient methodsFor: 'actions-hooks'!
releaseSocket
	socket destroy.! !

!SoapAbstractSocketClient methodsFor: 'actions-hooks'!
sendBySocket
	self subclassResponsibility! !


!SoapAbstractSocketClient methodsFor: 'accessing' stamp: 'MU 11/19/2002 12:22'!
host

	^host! !

!SoapAbstractSocketClient methodsFor: 'accessing' stamp: 'MU 11/19/2002 12:22'!
host: aValue

	host := aValue! !

!SoapAbstractSocketClient methodsFor: 'accessing' stamp: 'MU 11/19/2002 12:22'!
port

	^port! !

!SoapAbstractSocketClient methodsFor: 'accessing' stamp: 'MU 11/19/2002 12:22'!
port: aValue

	port := aValue! !

!SoapAbstractSocketClient methodsFor: 'accessing' stamp: 'MU 11/19/2002 12:22'!
reqURI

	^reqURI! !

!SoapAbstractSocketClient methodsFor: 'accessing' stamp: 'MU 11/19/2002 12:22'!
reqURI: aValue

	reqURI := aValue! !

!SoapAbstractSocketClient methodsFor: 'accessing' stamp: 'MU 11/19/2002 12:22'!
socket

	^socket! !

!SoapAbstractSocketClient methodsFor: 'accessing' stamp: 'MU 11/19/2002 12:23'!
socket: aValue

	socket := aValue! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

SoapAbstractSocketClient class
	instanceVariableNames: 'showLog resolveTimeout connectionTimeout connectionPool '!

!SoapAbstractSocketClient class methodsFor: 'class initialization'!
initConnectionPool
	connectionPool := IdentityDictionary new! !

!SoapAbstractSocketClient class methodsFor: 'class initialization' stamp: 'mu 9/18/2002 17:48'!
initHostAddressCache
	"SoapAbstractSocketClient initHostAddressCache"
	HostAddressCache := Dictionary new.
	HostAddressCache at: 'localhost' put: (ByteArray with: 127 with: 0 with: 0 with: 1)
	! !

!SoapAbstractSocketClient class methodsFor: 'class initialization'!
initialize
	"SoapAbstractSocketClient initialize"
	showLog := false.
	resolveTimeout := 20.

	self initHostAddressCache.
	self initConnectionPool.
	
	Smalltalk addToStartUpList: self.
	Smalltalk addToShutDownList: self! !

!SoapAbstractSocketClient class methodsFor: 'class initialization' stamp: 'mu 9/18/2002 17:38'!
obsolete
	Smalltalk removeFromStartUpList: self.
	Smalltalk removeFromShutDownList: self! !


!SoapAbstractSocketClient class methodsFor: 'setting' stamp: 'mu 10/9/2002 16:53'!
connectionTimeout
	connectionTimeout isNil ifTrue: [connectionTimeout := 30].
	^connectionTimeout! !

!SoapAbstractSocketClient class methodsFor: 'setting' stamp: 'mu 10/9/2002 16:47'!
connectionTimeout: timeout
	"SoapAbstractSocketClient connectionTimeout: 20"
	connectionTimeout := timeout! !

!SoapAbstractSocketClient class methodsFor: 'setting' stamp: 'mu 9/19/2002 18:40'!
resolveTimeout
	resolveTimeout isNil ifTrue: [resolveTimeout := 20].
	^resolveTimeout! !

!SoapAbstractSocketClient class methodsFor: 'setting' stamp: 'mu 9/19/2002 18:47'!
resolveTimeout: timeout
	"SoapAbstractSocketClient resolveTimeout: 20"
	resolveTimeout := timeout! !

!SoapAbstractSocketClient class methodsFor: 'setting' stamp: 'mu 9/18/2002 18:21'!
showLog
	^showLog! !

!SoapAbstractSocketClient class methodsFor: 'setting' stamp: 'mu 9/18/2002 18:22'!
showLog: aBoolean
	"SoapHttpClient showLog: true"
	"SoapHttpClient showLog: false"
	"SoapSqtpClient showLog: true"
	"SoapSqtpClient showLog: false"
	showLog := aBoolean! !

!SoapAbstractSocketClient class methodsFor: 'setting' stamp: 'MU 9/18/2002 12:22'!
useHostAddressCache
	UseHostAddressCache isNil ifTrue: [UseHostAddressCache := false].
	^UseHostAddressCache! !

!SoapAbstractSocketClient class methodsFor: 'setting' stamp: 'mu 9/18/2002 19:09'!
useHostAddressCache: aBoolean
	"SoapAbstractSocketClient useHostAddressCache: true"
	self initHostAddressCache.
	UseHostAddressCache := aBoolean! !


!SoapAbstractSocketClient class methodsFor: 'start up/shut down' stamp: 'mu 9/18/2002 17:37'!
shutDown
	self initHostAddressCache! !

!SoapAbstractSocketClient class methodsFor: 'start up/shut down' stamp: 'mu 9/18/2002 17:37'!
startUp
	self initHostAddressCache! !


!SoapAbstractSocketClient class methodsFor: 'accessing'!
connectionPool
	connectionPool isNil ifTrue: [self initConnectionPool].
	^connectionPool! !

!SoapAbstractSocketClient class methodsFor: 'accessing' stamp: 'mu 9/18/2002 17:32'!
hostAddressCache
	HostAddressCache isNil ifTrue: [self initHostAddressCache].
	^HostAddressCache! !


SoapImplError subclass: #SoapCannotConnectToServer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOAP-ClientBase'!


SoapImplError subclass: #SoapCannotResolveServer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOAP-ClientBase'!


Object subclass: #SoapConnectorFactory
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOAP-ClientBase'!

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

SoapConnectorFactory class
	instanceVariableNames: ''!

!SoapConnectorFactory class methodsFor: 'instance creation' stamp: 'mu 5/1/2003 18:01'!
createFrom: anTransportSymbol
	^SoapConnectorMap at: anTransportSymbol! !


SoapConfigurationMap subclass: #SoapConnectorMap
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOAP-ClientBase'!

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

SoapConnectorMap class
	instanceVariableNames: ''!

!SoapConnectorMap class methodsFor: 'actions' stamp: 'mu 5/1/2003 17:39'!
at: aSymbol
	| cls |
	cls := super at: aSymbol.
	^cls isNil ifTrue:[SoapNoSuchConnector signal: aSymbol] ifFalse:[cls new]! !


!SoapConnectorMap class methodsFor: 'factory' stamp: 'mu 5/1/2003 17:38'!
rootImplClassName
	^#SoapAbstractConnector! !


SoapAbstractConnector subclass: #SoapLocalRpcConnector
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOAP-ClientBase'!

!SoapLocalRpcConnector methodsFor: 'actions' stamp: 'MU 8/26/2002 23:47'!
send: aSoapEnvelope to: anSoapLocatorObject with: aSoapContext 
	| sercviceHandler returnEnv message |
	"ignore the locator and just invoke a local call"
	
	self debugSend: aSoapEnvelope to: anSoapLocatorObject with: aSoapContext.
	
	sercviceHandler := SoapServiceHandler default.
	returnEnv := sercviceHandler dispatch: aSoapEnvelope with: aSoapContext.
	message := SoapMessage envelope: returnEnv.
	self debugReturn: message.
	^message! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

SoapLocalRpcConnector class
	instanceVariableNames: 'debugMode '!

!SoapLocalRpcConnector class methodsFor: 'class initialization' stamp: 'MU 10/5/2002 02:27'!
initialize
	self initializeAfterLoad! !


!SoapLocalRpcConnector class methodsFor: 'constants' stamp: 'mu 8/28/2002 17:03'!
transportName
	^#localRpc! !


SoapNoSuchClass subclass: #SoapNoSuchConnector
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOAP-ClientBase'!


Object subclass: #SoapTcpLocator
	instanceVariableNames: 'host port '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOAP-ClientBase'!

!SoapTcpLocator methodsFor: 'accessing' stamp: 'mu 6/2/2001 02:27'!
host

	^host! !

!SoapTcpLocator methodsFor: 'accessing' stamp: 'MU 11/19/2002 11:55'!
host: hostnameOrIpAddressString

	host := hostnameOrIpAddressString asLowercase! !

!SoapTcpLocator methodsFor: 'accessing' stamp: 'mu 6/2/2001 02:27'!
port

	^port! !

!SoapTcpLocator methodsFor: 'accessing' stamp: 'mu 6/2/2001 02:27'!
port: aValue

	port := aValue! !


!SoapTcpLocator methodsFor: 'printing' stamp: 'mu 7/20/2001 02:02'!
printOn: aStream
	super printOn: aStream.
	aStream nextPutAll: ' host: '.
	aStream print: self host.
	aStream nextPutAll: ' port: '.
	aStream print: self port.! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

SoapTcpLocator class
	instanceVariableNames: ''!

!SoapTcpLocator class methodsFor: 'instance creation' stamp: 'mu 7/20/2001 01:59'!
host: hostnameString port: portNumberInteger
	^self new host: hostnameString; port: portNumberInteger! !

SoapAbstractConnector initialize!
SoapAbstractSocketClient initialize!
SoapLocalRpcConnector initialize!