SoapDataBinder subclass: #SoapAbstractDecoder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOAP-Decoding'!

!SoapAbstractDecoder methodsFor: 'actions' stamp: 'mu 8/26/2001 00:28'!
decodeXmlElement: aSoapWrapElement
	self subclassResponsibility! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

SoapAbstractDecoder class
	instanceVariableNames: ''!

!SoapAbstractDecoder class methodsFor: 'class initialization' stamp: 'MU 3/31/2002 20:55'!
initTypeDict
	"Apparently this mapping is not so sophisticted. "
	"##TODO: Implement more serious mapping method "
	| pref |
	typeDict := super initTypeDict.
	pref := SoapConstants xsdPrefixColon.
	#(
	#(#any #String)
	#(#anyType #String)
	#(#time #String)
	#(#boolean #Boolean)
	#(#float #Float)
	#(#double #Double)
	#(#integer #Integer)
	#(#positiveInteger #LargePositiveInteger )
	#(#negativeInteger #LargeNegativeInteger )
	#(#int #Integer)
	#(#string #String)
	) do:[:each | typeDict at: (pref, each first) asSymbol put: each last].
	^typeDict! !


SoapAbstractDecoder subclass: #SoapDecoder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOAP-Decoding'!

!SoapDecoder methodsFor: 'actions' stamp: 'MU 4/27/2002 02:19'!
decodeXmlElement: aSoapWrapElement 
	self class useXsiNullAttribute
		ifTrue: [^ self decodeNilXmlElement: aSoapWrapElement].
	aSoapWrapElement isArrayType
		ifTrue: [^ self decodeArrayXmlElement: aSoapWrapElement].
	^ aSoapWrapElement children isEmpty
		ifTrue: [self decodePrimitiveTyped: aSoapWrapElement xsiType value: aSoapWrapElement value]
		ifFalse:[self decodeComplexXmlElement: aSoapWrapElement]! !


!SoapDecoder methodsFor: 'private' stamp: 'mu 9/16/2003 11:40'!
decodeArrayXmlElement: aSoapWrapElement 
	| sz array |
	sz := aSoapWrapElement size.
	array := Array new: sz.
	1 to: aSoapWrapElement children size
		do: [:idx | 
			| itemElem | 
			itemElem := aSoapWrapElement children	at: idx	ifAbsent: [].
			itemElem isNil
				ifFalse: [| itemType decodedItemValue | 
					itemType := aSoapWrapElement arrayItemType.
					decodedItemValue := ((itemType = SoapConstants xsdAnyType
										or: [itemElem isArrayType])
									or: [itemElem children isEmpty not])
								ifTrue: [self decodeXmlElement: itemElem]
								ifFalse: [self decodePrimitiveTyped: itemType value: itemElem value].
					array at: idx put: decodedItemValue]].
	^ array! !

!SoapDecoder methodsFor: 'private' stamp: 'MU 4/28/2002 01:57'!
decodeComplexXmlElement: aSoapXmlElement 
	| stClass |
	stClass := self class environment
				at: aSoapXmlElement xsiType
				ifAbsent: [].
	stClass isNil
		ifTrue: [| structDict | 
			structDict := Dictionary new.
			aSoapXmlElement children
				do: [:eachChild | structDict
						at: (SoXmlUtil localNameFrom: eachChild name)
						put: (self decodeXmlElement: eachChild)].
			^ structDict]
		ifFalse: [| structInst allInstVarNames | 
			structInst := stClass new.
			allInstVarNames := stClass allInstVarNames.
			aSoapXmlElement children
				do: [:eachChild | structInst
						instVarAt: (allInstVarNames
								indexOf: (SoXmlUtil localNameFrom: eachChild name))
						put: (self decodeXmlElement: eachChild)].
			^ structInst]! !

!SoapDecoder methodsFor: 'private' stamp: 'MU 3/20/2002 00:54'!
decodeNilXmlElement: aSoapWrapElement
	self class useXsiNullAttribute ifTrue: [
		| xsiNull |
		xsiNull := aSoapWrapElement getAttributeNamed: (SoapConstants xsiNil).
		(xsiNull = '1' or: [xsiNull = 'true']) ifTrue:[^nil].
	].
	
	
	
	
	! !

!SoapDecoder methodsFor: 'private' stamp: 'MU 5/29/2002 01:15'!
decodePrimitiveTyped: xsiTypeName value: value 
	"quick hack - should see both prefix url and type"
	| xsiTypeShortName stClsName stClass val  |
	xsiTypeName isNil ifTrue:  [^value].
	xsiTypeShortName := SoXmlUtil localNameFrom: xsiTypeName.
	
	(xsiTypeShortName = SoapConstants string) ifTrue: [^value isNil ifTrue:[''] ifFalse:[value]].
	(xsiTypeShortName = SoapConstants time) ifTrue: [^value]. "quick hack"
	((xsiTypeShortName = SoapConstants base64Binary) or: [xsiTypeShortName = SoapConstants base64]) ifTrue:[^(Base64MimeConverter mimeDecodeToBytes: (value readStream)) contents].
	
	stClsName := self typeDict at: xsiTypeName asSymbol ifAbsent:[ SoapDecodingError signal: xsiTypeName, '''s decoding is not supported by ', self class printString].
	stClass := self class environment at: stClsName.
	val := stClass readFrom: (ReadStream on: value).
	^val! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

SoapDecoder class
	instanceVariableNames: 'useXsiNullAttribute '!

!SoapDecoder class methodsFor: 'class initialization' stamp: 'mu 2/6/2002 01:44'!
initTypeDict
	"Apparently this mapping is not so sophisticted. "
	"##TODO: Implement more serious mapping method "
	| pref |
	typeDict := super initTypeDict.
	pref := SoapConstants xsdPrefixColon.
	#(
	#(#double #Float)
	#(#base64Binary #ByteArray)
	) do:[:each | typeDict at: (pref, each first) asSymbol put: each last]. "for Poor Squeak!!" 
	^typeDict! !

!SoapDecoder class methodsFor: 'class initialization' stamp: 'mu 1/1/2002 20:59'!
initialize
	"SoapDecoder initialize"
	super initialize.
	useXsiNullAttribute := false.

	
	! !


!SoapDecoder class methodsFor: 'setting' stamp: 'mu 1/1/2002 19:32'!
useXsiNullAttribute
	^useXsiNullAttribute! !

!SoapDecoder class methodsFor: 'setting' stamp: 'mu 1/1/2002 19:33'!
useXsiNullAttribute: aBoolean
	useXsiNullAttribute := aBoolean! !


SoapDataBindingFactory subclass: #SoapDecoderFactory
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOAP-Decoding'!

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

SoapDecoderFactory class
	instanceVariableNames: ''!

!SoapDecoderFactory class methodsFor: 'default values' stamp: 'MU 10/15/2002 00:01'!
initDataBindingDict
	self dataBindingDict at: SoapConstants nullEncURI put: SoapNullDecoder.
	self dataBindingDict at: SoapConstants soapEncURI put: SoapDecoder.
	self dataBindingDict at: SoapConstants soapSqEncURI put: SoapSqDecoder.! !


!SoapDecoderFactory class methodsFor: 'accessing-error' stamp: 'mu 10/6/2001 20:39'!
noSuchDataBinder
	^SoapNoSuchDecoder
	! !


SoapDatabindingError subclass: #SoapDecodingError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOAP-Decoding'!


SoapNoSuchDataBinder subclass: #SoapNoSuchDecoder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOAP-Decoding'!


SoapAbstractDecoder subclass: #SoapNullDecoder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOAP-Decoding'!

!SoapNullDecoder methodsFor: 'actions' stamp: 'MU 10/14/2002 23:46'!
decodeXmlElement: aSoapWrapElement

	^aSoapWrapElement value! !


SoapAbstractDecoder subclass: #SoapSqDecoder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOAP-Decoding'!

!SoapSqDecoder methodsFor: 'actions' stamp: 'MU 3/25/2002 00:24'!
decodeXmlElement: aSoapWrapElement
	| xsiType stClsName stClass val |
	xsiType := aSoapWrapElement xsiType.
	(xsiType isNil or:[xsiType = (SoapConstants xsdPrefixColon, 'string')]) ifTrue: [^aSoapWrapElement value].
	xsiType = (SoapConstants sqEncPrefixColon, self defaultObjectDecoderClass name) ifTrue: [^self decodeObjectXmlElement: aSoapWrapElement].
	stClsName := self typeDict at: xsiType asSymbol ifAbsent:[ SoapDecodingError signal: xsiType, '''s decoding is not supported by ', self class printString].
	stClass := self class environment at: stClsName.
	val := stClass readFrom: (ReadStream on: aSoapWrapElement value).
	^val! !


!SoapSqDecoder methodsFor: 'private' stamp: 'mu 12/28/2001 02:04'!
decodeObjectXmlElement: aSoapWrapElement
	^self objectFromString: aSoapWrapElement value.
	! !

!SoapSqDecoder methodsFor: 'private' stamp: 'mu 12/28/2001 01:58'!
objectFromString: aString
	| byteStr oStr |
	byteStr := (Base64MimeConverter mimeDecodeToBytes: (ReadStream on: aString)).
	oStr := self defaultObjectDecoderClass on: byteStr.
	^oStr next! !


!SoapSqDecoder methodsFor: 'defaults' stamp: 'mu 12/28/2001 18:24'!
defaultObjectDecoderClass
	^ReferenceStream! !

SoapDecoder initialize!