SoapImplError subclass: #SoapSqtpGenericError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOAP-Sqtp'!


SoapSqtpGenericError subclass: #SoapSqtpInvalidHeader
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOAP-Sqtp'!


SoapSqtpGenericError subclass: #SoapSqtpInvalidSocketConnection
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOAP-Sqtp'!


Object subclass: #SoapSqtpProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOAP-Sqtp'!
!SoapSqtpProtocol commentStamp: 'MU 10/5/2002 01:13' prior: 0!
SQTP (Squeak Transport Protocol) is a tiny transport protocol for Squeak.
I represent that protocol.

SQTP message consists of two parts: Header and Body.

Header format is as follows: '{SQTP}{versionString} {command} #{encoder} *{sizeOfBody}:'.
For example, 'SQTP01 REQUEST #ReferenceStream *10486589:' means SQTP version 01 message whose command is REQUEST and whose encoder is #ReferenceStream.

Two commands are supported:
 -REQUEST - send some request object to server.
 -RETURN - return some return object to client.

Body is binary data which is encoded in some encoder specified in Header's encoder field.

---
MU 10/5/2002 00:59 !


"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

SoapSqtpProtocol class
	instanceVariableNames: 'platform waitTimeout '!

!SoapSqtpProtocol class methodsFor: 'class initialization' stamp: 'MU 11/19/2002 17:00'!
initialize
	"SoapSqtpProtocol initialize"
	
	Smalltalk addToStartUpList: self.
	
	self startUp
	! !

!SoapSqtpProtocol class methodsFor: 'class initialization' stamp: 'MU 11/19/2002 16:55'!
obsolete
	Smalltalk removeFromStartUpList: self.
! !


!SoapSqtpProtocol class methodsFor: 'start up/shut down' stamp: 'mu 5/1/2003 19:23'!
startUp
	platform := Smalltalk platformName asLowercase asSymbol.
	! !


!SoapSqtpProtocol class methodsFor: 'accessing' stamp: 'mu 5/1/2003 19:23'!
waitTimeout
	waitTimeout isNil ifTrue: [waitTimeout := 5].
	^waitTimeout! !

!SoapSqtpProtocol class methodsFor: 'accessing' stamp: 'mu 5/1/2003 19:24'!
waitTimeout: seconds
	"SoapSqtpProtocol waitTimeout: 15"
	waitTimeout := seconds! !


!SoapSqtpProtocol class methodsFor: 'actions' stamp: 'MU 11/19/2002 14:40'!
getObjectFrom: socket 
	| data object | 
	data := self getDataFrom: socket.
	object := self objectFromBinary: data.
	^object! !

!SoapSqtpProtocol class methodsFor: 'actions' stamp: 'MU 10/5/2002 00:00'!
sendRequestObject: object by: socket
	^self sendObject: object by: socket command: #REQUEST
	! !

!SoapSqtpProtocol class methodsFor: 'actions' stamp: 'MU 10/5/2002 00:00'!
sendReturnObject: object by: socket
	^self sendObject: object by: socket command: #RETURN
	! !


!SoapSqtpProtocol class methodsFor: 'private' stamp: 'mu 9/8/2003 00:47'!
getDataFrom: socket
	
	[| firstRead headerDivider headerBufferStream header bodySize encoded dataComing |

	(self isSafeSocket: socket) ifFalse: [SoapSqtpInvalidSocketConnection signal].
	
	[socket waitForDataFor: self waitTimeout] on: ConnectionClosed, ConnectionTimedOut 
		do: [:ex | SoapSqtpInvalidSocketConnection signal: ex messageText].
	
	firstRead := socket receiveData.
	headerDivider := firstRead indexOf: $:.
	headerDivider = 0 ifTrue: [SoapSqtpInvalidHeader signal.].
	headerBufferStream := firstRead readStream.
	header := headerBufferStream next: headerDivider.
	bodySize := (header copyFrom: (header indexOf: $*) + 1 to: header size - 1 ) asNumber. 
	encoded := WriteStream on: (String new: bodySize).
	encoded nextPutAll: headerBufferStream upToEnd.
	
	[(self isSafeSocket: socket) and:[bodySize > encoded position]]
		whileTrue: [encoded nextPutAll: socket receiveData].
	^encoded contents
	] on: Error do: [:ex | 
	(self isSocketPrimitiveFailed: ex ) ifTrue: [ex resignalAs: SoapSqtpSocketPrimitiveFailed new] ifFalse: [ex pass]]! !

!SoapSqtpProtocol class methodsFor: 'private' stamp: 'mu 11/20/2002 13:45'!
sendData: data by: socket in: encoder command: command 
	
	[| headerStream header sentSize toSendSize sendData |
	(self isSafeSocket: socket) ifFalse: [SoapSqtpInvalidSocketConnection signal].

	headerStream := WriteStream on: (String new: 80).
	headerStream nextPutAll: 'SQTP01'; space.
	headerStream nextPutAll: command; space.
	headerStream nextPutAll: '#ReferenceStream';space.
	"Currently encoder is ReferenceStream only"
	headerStream nextPutAll: '*' , data size printString , ':'.

	header := headerStream contents.
	sentSize := 0.
	toSendSize := header size + data size.
	sendData := header , data.
	[sentSize < toSendSize]
		whileTrue: [self
				safeSocketDo: [sentSize := sentSize
								+ (socket sendSomeData: sendData startIndex: sentSize + 1)]
				with: socket]
	] on: Error do: [:ex | 
	(self isSocketPrimitiveFailed: ex ) ifTrue: [ex resignalAs: SoapSqtpSocketPrimitiveFailed new] ifFalse: [ex pass]]! !

!SoapSqtpProtocol class methodsFor: 'private' stamp: 'MU 11/19/2002 14:41'!
sendObject: object by: socket command: command 
	| data |
	data := self binaryFromObject: object.
	self
		sendData: data
		by: socket
		in: #ReferenceStream
		command: command! !


!SoapSqtpProtocol class methodsFor: 'private-socket' stamp: 'MU 11/19/2002 17:16'!
isSafeSocket: socket 
	socket isValid not	ifTrue: [^ false].
	platform == #'mac os' ifFalse: [
	socket isUnconnected	ifTrue: [^ false].
	socket isOtherEndClosed ifTrue: [^ false]].
	^ true! !

!SoapSqtpProtocol class methodsFor: 'private-socket' stamp: 'mu 11/20/2002 13:38'!
isSocketPrimitiveFailed: exception 
	^ exception messageText = 'a primitive has failed'! !

!SoapSqtpProtocol class methodsFor: 'private-socket' stamp: 'MU 11/19/2002 14:18'!
safeSocketDo: aBlock with: socket 
	(self isSafeSocket: socket)
		ifTrue: [[^ aBlock value]
				on: Error
				do: [:ex | 
					ex inspect.
					ex pass]]
		ifFalse: [SoapSqtpInvalidSocketConnection signal]! !


!SoapSqtpProtocol class methodsFor: 'private-serialization' stamp: 'MU 11/19/2002 14:10'!
binaryFromObject: anObject 
	| file |
	"Currently ReferenceStream only"
	file := RWBinaryOrTextStream
				on: (ByteArray new: 5000).
	file binary.
	(ReferenceStream on: file)
		nextPut: anObject.
	file close.
	^ file contents! !

!SoapSqtpProtocol class methodsFor: 'private-serialization' stamp: 'MU 11/19/2002 14:10'!
objectFromBinary: bytes 
	"Currently ReferenceStream only"
	| object |
	bytes isNil ifTrue: [^nil].
	object := ReferenceStream unStream: bytes.
	^ object! !


SoapSqtpGenericError subclass: #SoapSqtpSocketPrimitiveFailed
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOAP-Sqtp'!

SoapSqtpProtocol initialize!