TestCase subclass: #SoapCallTestCaseForSixx
	instanceVariableNames: 'originalTransports '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOAP-TestCases-SIXX'!

!SoapCallTestCaseForSixx methodsFor: 'testing' stamp: 'MU 8/1/2002 01:34'!
testComplexSixxEncoding
	| call pa1 pa2 env methodElem arg1 arg2 |
	self should:[Smalltalk includesKey: #SixxWriteStream].
	call := SoapCall new.
	call transport: #localRpc.
	call targetObjectURI: 'urn:auauObject'.
	call encoding: #soapSixxEncoding.
	call methodName: 'auau'.
	pa1 := (SoapVariable name: #arg1 value: 1).
	pa2 := (SoapVariable name: #arg2 value: #two) encoding: #soapSixxEncoding.
	call addParameter: pa1.
	call addParameter: pa2.
	env := call buildEnvelope.
	
	self should: [env name = 'env:Envelope'].
	self should: [env body name = 'env:Body'].
	self should: [(env getAttributeNamed: 'env:encodingStyle') = SoapConstants soapSixxEncURI]. 
	methodElem := env body bodyEntries first.
	self should: [ methodElem name = 'm:auau'].
	arg1 := methodElem children at: 1.
	self should:[ arg1 name = 'arg1']. 
	self should:[ (arg1 children first getAttributeNamed: 'sixx.type') = 'SmallInteger']. 
	self should:[ (arg1 encodingStyleURIs first) = SoapConstants soapSixxEncURI]. 
	self should:[ (Object readSixxFrom: (arg1 children first xmlString)) = 1]. 
	arg2 := methodElem children at: 2.
	self should:[ arg2 name = 'arg2']. 
	self should:[ (arg2 children first getAttributeNamed: 'sixx.type') =  'Symbol']. ! !

!SoapCallTestCaseForSixx methodsFor: 'testing' stamp: 'mu 5/2/2003 18:47'!
testComplexSixxHttpCall
	| handlerName call resp |
	
	self should:[Smalltalk includesKey: #SixxWriteStream].
	
	SoapServerSetup startOnly: #(http).
	
	handlerName := SoapServiceHandler useTestServiceHandler ifTrue: [#test] ifFalse: [#default].
	(SoapServiceHandler named: handlerName ) add: self simpleAddService.
	call := (SoapCallEntry local: (SoapServerSetup defaultPortAt: #http)) newCall.
	call transport: #http.
	call encoding: #soapSixxEncoding.
	call methodName: 'add'.
	call addParameter: (SoapVariable name: #newElem).
	
	resp := call invokeWithValue: #symbol.
	self shouldnt:[resp isFault].
	self should:[resp returnValue = #symbol].
	
	resp := call invokeWithValue: Date today.
	self shouldnt:[resp isFault].
	self should:[resp returnValue = Date today].
	
	resp := call invokeWithValue: (OrderedCollection with: #(1 2 #('three' 3) with: Date today)).
	self shouldnt:[resp isFault].
	self should:[resp returnValue = (OrderedCollection with: #(1 2 #('three' 3) with: Date today))].
	
	(SoapServiceHandler named: handlerName) remove: self simpleAddService.
	! !

!SoapCallTestCaseForSixx methodsFor: 'testing' stamp: 'MU 8/1/2002 00:40'!
testSimpleSixxCall

	| call resp |
	self should:[Smalltalk includesKey: #SixxWriteStream].
	SoapServiceHandler default add: self simpleAddService.
	call := SoapCallEntry local newCall.
	call transport: #localRpc.
	call encoding: #soapSixxEncoding.
	call methodName: 'add'.
	call addParameter: (SoapVariable name: #newElem value: 1).
	
	resp := call invoke.
	
	self shouldnt:[resp isFault].
	self should:[resp returnValue = 1].! !


!SoapCallTestCaseForSixx methodsFor: 'fixtures' stamp: 'MU 8/1/2002 00:33'!
simpleAddService
	| ord serv |
	ord := OrderedCollection new.
	serv := SoapService implementor: ord selector: #add:.
	serv signature: (SoapServiceSignature name: 'add' paramNames: #(newElem)).
	^serv! !


!SoapCallTestCaseForSixx methodsFor: 'running' stamp: 'MU 4/18/2003 17:21'!
setUp
	originalTransports := SoapServerSetup runningTransportNames.
	SoapServerSetup startOnly: #(http) ! !

!SoapCallTestCaseForSixx methodsFor: 'running' stamp: 'MU 4/18/2003 17:21'!
tearDown
	
	SoapServerSetup startOnly: originalTransports! !


TestCase subclass: #SoapSixxDecoderTestCase
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOAP-TestCases-SIXX'!

!SoapSixxDecoderTestCase methodsFor: 'testing' stamp: 'MU 7/28/2002 01:35'!
testDecodeAnyArray
	| dec ret |
	dec := SoapSixxDecoder new.
	ret := dec decodeXmlElement: ((SoapVariable name: 'anyArr1' value: #(1 '2' 3.0 )) encoding: #soapSixxEncoding;
					 asXmlElement).
	self
		should: [ret = #(1 '2' 3.0 )]! !

!SoapSixxDecoderTestCase methodsFor: 'testing' stamp: 'MU 7/28/2002 01:38'!
testDecodeArrayWithEmptyString
	| dec ret |
	dec := SoapSixxDecoder new.
	ret := dec decodeXmlElement: ((SoapVariable name: 'anyArr1' value: #('' '2' 3.0 )) encoding: #soapSixxEncoding;
					 asXmlElement).
	self
		should: [ret = #('' '2' 3.0 )]! !

!SoapSixxDecoderTestCase methodsFor: 'testing' stamp: 'MU 7/28/2002 01:39'!
testDecodeBase64Binary
	| dec ret bytes |
	dec := SoapSixxDecoder new.
	bytes := ByteArray
				with: 1
				with: 2
				with: 3
				with: 4.
	ret := dec decodeXmlElement: ((SoapVariable name: 'binbin' value: bytes) encoding: #soapSixxEncoding;
					 asXmlElement).
	self
		should: [ret = bytes]! !

!SoapSixxDecoderTestCase methodsFor: 'testing' stamp: 'MU 7/28/2002 01:34'!
testDecodeBoolean
	| dec ret |
	dec := SoapSixxDecoder new.
	ret := dec decodeXmlElement: ((SoapVariable
					name: 'arg1'
					value: true) encoding: #soapSixxEncoding;
					 asXmlElement).
	self
		should: [ret = true]! !

!SoapSixxDecoderTestCase methodsFor: 'testing' stamp: 'MU 7/28/2002 01:33'!
testDecodeEmptyString
	| dec ret |
	dec := SoapSixxDecoder new.
	ret := dec decodeXmlElement: ((SoapVariable
					name: 'arg1'
					value: ''
					type: #String) encoding: #soapSixxEncoding;
					 asXmlElement).
	self
		should: [ret = '']! !

!SoapSixxDecoderTestCase methodsFor: 'testing' stamp: 'MU 7/28/2002 01:32'!
testDecodeFloatArray
	| dec ret |
	dec := SoapSixxDecoder new.
	ret := dec decodeXmlElement: ((SoapVariable name: 'floatArr1' value: #(1.1 2.2 3.3 )) encoding: #soapSixxEncoding;
					 asXmlElement).
	self
		should: [ret = #(1.1 2.2 3.3 )]! !

!SoapSixxDecoderTestCase methodsFor: 'testing' stamp: 'MU 7/28/2002 01:27'!
testDecodeInteger
	| dec ret |
	dec := SoapSixxDecoder new.
	ret := dec decodeXmlElement: ((SoapVariable
					name: 'arg1'
					value: 1234567) encoding: #soapSixxEncoding;
					 asXmlElement).
	self
		should: [ret = 1234567]! !

!SoapSixxDecoderTestCase methodsFor: 'testing' stamp: 'MU 7/28/2002 01:31'!
testDecodeIntegerArray
	| dec ret |
	dec := SoapSixxDecoder new.
	ret := dec decodeXmlElement: ((SoapVariable
					name: 'intArr1'
					value: #(1 2 3 )
					type: #Array) encoding: #soapSixxEncoding;
					 asXmlElement).
	self
		should: [ret = #(1 2 3 )]! !

!SoapSixxDecoderTestCase methodsFor: 'testing' stamp: 'MU 8/1/2002 00:30'!
testDecodeRecursiveData
	
	| dec ret secondDict |
	dec := SoapSixxDecoder new.
	ret := dec decodeXmlElement: ((SoapVariable name: 'recur1' value: self recursiveData) encoding: #soapSixxEncoding;
					 asXmlElement).
	
	self should:[(ret at: #firstArray) = #(1 2.0)].
	self should:[ret includesKey: #secondDict ].
	secondDict := ret at: #secondDict.
	self should:[(secondDict at: #key1 ) = 'foo'].
	self should:[(secondDict at: #key2 ) = 'bar'].
	
	self should:[(ret at: #thirdArray) = (Array with: #3 with: secondDict).].
	
	self should:[(ret at: #forthRecur) == ret].

	
	
		




	
	! !

!SoapSixxDecoderTestCase methodsFor: 'testing' stamp: 'MU 7/28/2002 01:22'!
testDecodeString
	| dec ret |
	dec := SoapSixxDecoder new.
	ret := dec decodeXmlElement: ((SoapVariable
					name: 'arg1'
					value: 'Smalltalk'
					type: #String) encoding: #soapSixxEncoding;
					 asXmlElement).
	self
		should: [ret = 'Smalltalk']! !

!SoapSixxDecoderTestCase methodsFor: 'testing' stamp: 'MU 7/28/2002 01:31'!
testDecodeStringArray
	| dec ret |
	dec := SoapSixxDecoder new.
	ret := dec decodeXmlElement: ((SoapVariable
					name: 'stringArr1'
					value: #('a' 'b' '<c>' )
					type: #Array) encoding: #soapSixxEncoding;
					 asXmlElement).
	self
		should: [ret = #('a' 'b' '<c>' )]! !


!SoapSixxDecoderTestCase methodsFor: 'fixtures' stamp: 'MU 8/1/2002 00:22'!
recursiveData
	| root dict |
	root := Dictionary new.
	dict := Dictionary new.
	dict at: #key1 put: 'foo'.
	dict at: #key2 put: 'bar'.
	root
		at: #firstArray
		put: (Array with: 1 with: 2.0).
	root at: #secondDict put: dict.
	root
		at: #thirdArray
		put: (Array with: 3 with: dict).
	root at: #forthRecur put: root.
	^ root! !


TestCase subclass: #SoapSixxEncoderTestCase
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOAP-TestCases-SIXX'!

!SoapSixxEncoderTestCase methodsFor: 'testing' stamp: 'MU 8/1/2002 01:17'!
testEncodeBoolean
	| enc ret |
	enc := SoapSixxEncoder new.

	ret := enc encodeSoapVariable: (SoapVariable name: 'auau' value: true).
	ret := ret children first.
	self should: [ (ret getAttributeNamed: 'sixx.type') = 'True'].
	self should: [ ret value = 'true'].
	
		
	
	
	
	
	
	! !

!SoapSixxEncoderTestCase methodsFor: 'testing' stamp: 'MU 8/1/2002 01:17'!
testEncodeByteArray
	| enc ret bytes |
	enc := SoapSixxEncoder new.

	bytes := ByteArray with: 1 with: 2 with: 3 with: 4.
	ret := enc encodeSoapVariable: (SoapVariable name: 'binbin' value: bytes).
	ret := ret children first.
	self should: [ (ret getAttributeNamed: 'sixx.type') = 'ByteArray'].
	self should: [ ret value =  '#( 1 2 3 4 )'].
	
	
	
	
	
	! !

!SoapSixxEncoderTestCase methodsFor: 'testing' stamp: 'MU 8/1/2002 01:18'!
testEncodeFloatArray
	| enc ret |
	enc := SoapSixxEncoder new.

	ret := enc encodeSoapVariable: (SoapVariable name: 'floatArr' value: #(1.0 2.3 3.4)).
	ret := ret children first.
	self should: [ (ret getAttributeNamed: #'sixx.type') = 'Array'].
	self should: [ (ret children at: 1) value = '1.0'].
	self should: [ (ret children at: 3) value = '3.4'].
	
	ret := enc encodeSoapVariable: (SoapVariable name: 'integarArr' value: #(4.5 5.6) type: #Array).
	ret := ret children first.
	self should: [ (ret getAttributeNamed: #'sixx.type') = 'Array'].
	self should: [ (ret children at: 1) value = '4.5'].
	self should: [ (ret children at: 2) value = '5.6'].
	
	ret := enc encodeSoapVariable: (SoapVariable name: 'integarArr' value: #()).
	ret := ret children first.
	self should: [ (ret getAttributeNamed: #'sixx.type') = 'Array'].
	self should: [ ret children isEmpty].
	
	
	
	
	! !

!SoapSixxEncoderTestCase methodsFor: 'testing' stamp: 'MU 8/1/2002 01:17'!
testEncodeInteger
	| enc ret |
	enc := SoapSixxEncoder new.

	ret := enc encodeSoapVariable: (SoapVariable name: 'auau' value: 1).
	ret := ret children first.
	self should: [ (ret getAttributeNamed: 'sixx.type') = 'SmallInteger'].
	self should: [ ret value = '1'].
	

	
	! !

!SoapSixxEncoderTestCase methodsFor: 'testing' stamp: 'MU 8/1/2002 01:19'!
testEncodeIntegerArray
	| enc ret |
	enc := SoapSixxEncoder new.

	ret := enc encodeSoapVariable: (SoapVariable name: 'integarArr' value: #(1 2 3)).
	ret := ret children first.
	self should: [ (ret getAttributeNamed: #'sixx.type') = 'Array'].
	self should: [ (ret children at: 1) value = '1'].
	self should: [ (ret children at: 3) value = '3'].
	
	ret := enc encodeSoapVariable: (SoapVariable name: 'integarArr' value: #(4 5) type: #Array).
	ret := ret children first.
	self should: [ (ret getAttributeNamed: #'sixx.type') = 'Array'].
	self should: [ (ret children at: 1) value = '4'].
	self should: [ (ret children at: 2) value = '5'].
	
	ret := enc encodeSoapVariable: (SoapVariable name: 'integarArr' value: #()).
	ret := ret children first.
	self should: [ (ret getAttributeNamed: #'sixx.type') = 'Array'].
	self should: [ ret children isEmpty].
	
	
	
	
	! !

!SoapSixxEncoderTestCase methodsFor: 'testing' stamp: 'MU 8/1/2002 01:18'!
testEncodeNil
	| enc ret |
	enc := SoapSixxEncoder new.
	
	ret := enc encodeSoapVariable: (SoapVariable name: 'auau' value: nil).
	ret := ret children first.
	self should: [ (ret getAttributeNamed: 'sixx.type') = 'UndefinedObject'].
	self should: [ ret value = 'nil'].
	
	
	
	
	! !

!SoapSixxEncoderTestCase methodsFor: 'testing' stamp: 'MU 8/1/2002 01:20'!
testEncodeSimpleStruct
	| enc ret dic retDictionary |
	enc := SoapSixxEncoder new.
	dic := Dictionary new.
	dic at: #varString put: 'hello'.
	dic at: #varInt put: 42.
	dic at: #varFloat put: 0.6.

	ret := enc encodeSoapVariable: (SoapVariable name: 'inputStruct' value: dic).
	self should:[ret name = 'inputStruct'].
	ret := ret children first.
	self should:[(ret getAttributeNamed: #'sixx.type') = 'Dictionary'].
	self should:[ret children size = 3].
	retDictionary := Object readSixxFrom: ret xmlString.
	self should:[(retDictionary at: #varString) = 'hello'].
	self should:[(retDictionary at: #varInt) = 42].
	self should:[(retDictionary at: #varFloat) = 0.6].
	
	
	
	
	
	
	! !

!SoapSixxEncoderTestCase methodsFor: 'testing' stamp: 'MU 8/1/2002 01:18'!
testEncodeString
	| enc ret |
	enc := SoapSixxEncoder new.
	
	ret := enc encodeSoapVariable: (SoapVariable name: 'auau' value: 'smalltalk').
	ret := ret children first.
	self should: [ (ret getAttributeNamed: 'sixx.type') = 'String'].
	self should: [ ret value = 'smalltalk'].
	
	
	
	
	
	! !

!SoapSixxEncoderTestCase methodsFor: 'testing' stamp: 'MU 8/1/2002 01:18'!
testEncodeStringArray
	| enc ret |
	enc := SoapSixxEncoder new.

	ret := enc encodeSoapVariable: (SoapVariable name: 'stringArr' value: #('a' 'b' 'c')).
	ret := ret children first.
	self should: [ (ret getAttributeNamed: #'sixx.type') = 'Array'].
	self should: [ (ret children at: 1) value = 'a'].
	self should: [ (ret children at: 3) value = 'c'].
	
	ret := enc encodeSoapVariable: (SoapVariable name: 'stringArr' value: #('' 'eee' nil) type: #Array).
	ret := ret children first.
	self should: [ (ret getAttributeNamed: #'sixx.type') = 'Array'].
	self should: [ (ret children at: 1) value = nil].
	self should: [ (ret children at: 2) value = 'eee'].
	self should: [ (ret children at: 3) value = 'nil'].
	
	ret := enc encodeSoapVariable: (SoapVariable name: 'stringArr' value: #()).
	ret := ret children first.
	self should: [ (ret getAttributeNamed: #'sixx.type') = 'Array'].
	self should: [ ret children isEmpty].
	
	
	
	
	! !
