TestCase subclass: #SoapSqtpCallTestCase
	instanceVariableNames: 'originalTransports '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOAP-TestCases-Sqtp'!

!SoapSqtpCallTestCase methodsFor: 'testing' stamp: 'mu 5/2/2003 18:48'!
testAnyArraySqtpCall
	| handlerName call resp |
	
	handlerName := SoapServiceHandler useTestServiceHandler ifTrue: [#test] ifFalse: [#default].
	(SoapServiceHandler named: handlerName ) add: self simpleAddService.
	call := (SoapCallEntry tcpHost: 'localhost' port: SoapSqtpSetup defaultPort) newCall.
	call transport: #sqtp.
	call methodName: 'add'.
	call addParameter: (SoapVariable name: #newElem value: #(1.0 'two' 3)).
	
	resp := call invoke.
	
	self shouldnt:[resp isFault].
	self should:[resp returnValue = #(1.0 'two' 3)].
	
	(SoapServiceHandler named: handlerName) remove: self simpleAddService.
	
	
	
	
	! !

!SoapSqtpCallTestCase methodsFor: 'testing' stamp: 'MU 9/28/2002 22:40'!
testBulkDataSqtpCall
	| longString call resp |
	SoapServiceHandler default  add: self simpleAddService.
	call := (SoapCallEntry tcpHost: 'localhost' port: SoapSqtpSetup defaultPort) newCall.
	call transport: #sqtp.
	call methodName: 'add'.

	longString := (String new: 1024 * 1024). "1000K"

	call addParameter: (SoapVariable name: #newElem value: longString).
	
	resp := call invoke.
	
	self shouldnt:[resp isFault].
	self should:[resp returnValue = longString].
	
	SoapServiceHandler default remove: self simpleAddService.
	
	
	
	! !

!SoapSqtpCallTestCase methodsFor: 'testing' stamp: 'mu 5/2/2003 17:00'!
testCannotConnectSqtpCall
	| notUsedPort call resp |
	notUsedPort := 9876.
	call := (SoapCallEntry tcpHost: 'localhost' port: notUsedPort) newCall.
	call transport: #sqtp.
	call methodName: 'helloWorld'.
	
	resp := call invoke.
	
	self should:[resp isFault].
	self should:[resp returnValue = nil].
	self should: [resp raiseException] raise: SoapCannotConnectToServer
	
	
	
	
	
	
	! !

!SoapSqtpCallTestCase methodsFor: 'testing' stamp: 'mu 5/2/2003 17:00'!
testCannotResolveSqtpCall
	| call resp |

	call := (SoapCallEntry tcpHost: 'foobarbazboo' port: 8823) newCall.
	call transport: #sqtp.
	call methodName: 'helloWorld'.
	
	resp := call invoke.
	
	self should:[resp isFault].
	self should:[resp returnValue = nil].
	self should: [resp raiseException] raise: SoapCannotResolveServer
	
	
	
	
	
	
	! !

!SoapSqtpCallTestCase methodsFor: 'testing' stamp: 'mu 5/2/2003 18:48'!
testComplexSqSqtpCall
	| handlerName call resp |

	handlerName := SoapServiceHandler useTestServiceHandler ifTrue: [#test] ifFalse: [#default].
	(SoapServiceHandler named: handlerName ) add: self simpleAddService.
	call := (SoapCallEntry tcpHost: 'localhost' port: SoapSqtpSetup defaultPort) newCall.
	call transport: #sqtp.
	call encoding: #soapSqEncoding.
	call methodName: 'add'.
	call addParameter: (SoapVariable name: #newElem).
	
	resp := call invokeWithValue: #symbol.
	self shouldnt:[resp isFault].
	self should:[resp returnValue = #symbol].
	
	resp := call invokeWithValue: Date today.
	self shouldnt:[resp isFault].
	self should:[resp returnValue = Date today].
	
	resp := call invokeWithValue: (OrderedCollection with: #(1 2 #('three' 3) with: Date today)).
	self shouldnt:[resp isFault].
	self should:[resp returnValue = (OrderedCollection with: #(1 2 #('three' 3) with: Date today))].
	
	(SoapServiceHandler named: handlerName) remove: self simpleAddService.
	
	
	
	
	! !

!SoapSqtpCallTestCase methodsFor: 'testing' stamp: 'mu 5/2/2003 18:49'!
testIntegerArraySqtpCall
	| handlerName call resp |
	
	handlerName := SoapServiceHandler useTestServiceHandler ifTrue: [#test] ifFalse: [#default].
	(SoapServiceHandler named: handlerName ) add: self simpleAddService.
	call := (SoapCallEntry tcpHost: 'localhost' port: SoapSqtpSetup defaultPort) newCall.
	call transport: #sqtp.
	call methodName: 'add'.
	call addParameter: (SoapVariable name: #newElem value: #(1 2 3)).
	
	resp := call invoke.
	
	self shouldnt:[resp isFault].
	self should:[resp returnValue = #(1 2 3)].
	
	(SoapServiceHandler named: handlerName) remove: self simpleAddService.
	
	
	
	
	! !

!SoapSqtpCallTestCase methodsFor: 'testing' stamp: 'mu 5/2/2003 18:49'!
testSimpleSqSqtpCall
	| handlerName call resp |
	
	handlerName := SoapServiceHandler useTestServiceHandler ifTrue: [#test] ifFalse: [#default].
	(SoapServiceHandler named: handlerName ) add: self simpleAddService.
	call := (SoapCallEntry tcpHost: 'localhost' port: SoapSqtpSetup defaultPort) newCall.
	call transport: #sqtp.
	call encoding: #soapSqEncoding.
	call methodName: 'add'.
	call addParameter: (SoapVariable name: #newElem value: 1).
	
	resp := call invoke.
	
	self shouldnt:[resp isFault].
	self should:[resp returnValue = 1].
	
	(SoapServiceHandler named: handlerName) remove: self simpleAddService.
	
	
	
	
	! !

!SoapSqtpCallTestCase methodsFor: 'testing' stamp: 'MU 8/27/2002 18:53'!
testSimpleSqtpCall
	| call resp |
	SoapServiceHandler default  add: self simpleAddService.
	call := (SoapCallEntry tcpHost: 'localhost' port: SoapSqtpSetup defaultPort) newCall.
	call transport: #sqtp.
	call methodName: 'add'.
	call addParameter: (SoapVariable name: #newElem value: 1).
	
	resp := call invoke.
	
	self shouldnt:[resp isFault].
	self should:[resp returnValue = 1].
	
	SoapServiceHandler default remove: self simpleAddService.
	
	
	
	! !

!SoapSqtpCallTestCase methodsFor: 'testing' stamp: 'mu 5/2/2003 18:49'!
testSimpleSqtpCallWithError
	| handlerName call resp |
	
	handlerName := SoapServiceHandler useTestServiceHandler ifTrue: [#test] ifFalse: [#default].
	(SoapServiceHandler named: handlerName ) add: self simpleAddService.
	call := (SoapCallEntry tcpHost: 'localhost' port: SoapSqtpSetup defaultPort )newCall.
	call methodName: 'add'.
	call transport: #sqtp.
	call addParameter: (SoapVariable name: #badArg1 value: 1).

	resp := call invoke.
	self should:[resp isFault].
	self should:[resp returnValue = nil].
	self should:[resp raiseException] raise: SoapInvalidParamName.
	
	
	(SoapServiceHandler named: handlerName) remove: self simpleAddService.
	
	
	
	
	! !


!SoapSqtpCallTestCase methodsFor: 'fixtures' stamp: 'MU 8/26/2002 18:18'!
simpleAddService
	| ord serv |
	ord := OrderedCollection new.
	serv := SoapService implementor: ord selector: #add:.
	serv signature: (SoapServiceSignature name: 'add' paramNames: #(newElem)).
	^serv! !


!SoapSqtpCallTestCase methodsFor: 'running' stamp: 'MU 4/18/2003 17:21'!
setUp
	originalTransports  := SoapServerSetup runningTransportNames.
	SoapServerSetup startOnly: #(sqtp) ! !

!SoapSqtpCallTestCase methodsFor: 'running' stamp: 'MU 4/18/2003 17:21'!
tearDown
	
	SoapServerSetup startOnly: originalTransports ! !
